# -*- coding:utf-8 -*-
"""
市场类
Created on 2018/12/26
@author: TabQ
@group : gugu
@contact: 16621596@qq.com
"""

import pandas as pd
from pandas.compat import StringIO
import re
import json
import numpy as np
from gugu.base import Base, cf


class MarketData(Base):
    def index(self):
        """
        获取大盘指数行情
        return
        -------
          DataFrame or list: [{'code':, 'name':, ...}, ...]
              code:指数代码
              name:指数名称
              change:涨跌幅
              open:开盘价
              preclose:昨日收盘价
              close:收盘价
              high:最高价
              low:最低价
              volume:成交量(手)
              amount:成交金额（亿元）
        """
        self._data = pd.DataFrame()
        
        # http://hq.sinajs.cn/rn=xppzh&list=sh000001,sh000002,sh000003,sh000008,sh000009,sh000010,sh000011,sh000012,sh000016,sh000017,sh000300,sh000905,sz399001,sz399002,sz399003,sz399004,sz399005,sz399006,sz399008,sz399100,sz399101,sz399106,sz399107,sz399108,sz399333,sz399606
        request = self._session.get( cf.INDEX_URL, timeout=10 )
        request.encoding = 'gbk'
        text = request.text.replace('var hq_str_', '')
        text = text.replace('";', '').replace('"', '').replace('=', ',')
        text = '%s%s'%(cf.INDEX_HEADER, text)
        
        self._data = pd.read_csv(StringIO(text), sep=',', thousands=',')
        self._data['change'] = (self._data['close'] / self._data['preclose'] - 1 ) * 100
        self._data['amount'] = self._data['amount'] / 100000000
        self._data['change'] = self._data['change'].map(cf.FORMAT)
        self._data['amount'] = self._data['amount'].map(cf.FORMAT4)
        self._data = self._data[cf.INDEX_COLS]
        self._data['code'] = self._data['code'].map(lambda x:str(x).zfill(6))
        self._data['change'] = self._data['change'].astype(float)
        self._data['amount'] = self._data['amount'].astype(float)
        
        return self._result()
    
    
    def latest(self):
        """
        一次性获取最近一个日交易日所有股票的交易数据
        return
        -------
          DataFrame or list: [{'code':, 'name':, ...}, ...]
		code：代码
		name:名称
		changepercent:涨跌幅
		trade:现价
		open:开盘价
		high:最高价
		low:最低价
		settlement:昨日收盘价
		volume:成交量
		turnoverratio:换手率
		amount:成交量
		per:市盈率
		pb:市净率
		mktcap:总市值
		nmc:流通市值
        """
        self._data = pd.DataFrame()
        
        self._writeHead()
        
        self._data = self.__handleLatest(1)
        if self._data is not None:
            for i in range(2, cf.PAGE_NUM[0]):
                newData = self.__handleLatest(i)
                self._data = self._data.append(newData, ignore_index=True)
        
        return self._result()
    
    
    def __handleLatest(self, pageNum=1):
        """
        处理当日行情分页数据，格式为json
        Parameters
        ------
            pageNum:页码
        return
        -------
            DataFrame 当日所有股票交易数据(DataFrame)
        """
        self._writeConsole()
        # http://vip.stock.finance.sina.com.cn/quotes_service/api/json_v2.php/Market_Center.getHQNodeData?num=80&sort=code&asc=0&node=hs_a&symbol=&_s_r_a=page&page=1
        request = self._session.get( cf.LATEST_URL % pageNum, timeout=10 )
        if self._PY3:
            request.encoding = 'gbk'
        text = request.text
        if text == 'null':
            return None
        
        reg = re.compile(r'\,(.*?)\:')
        text = reg.sub(r',"\1":', text)
        text = text.replace('"{symbol', '{"symbol')
        text = text.replace('{symbol', '{"symbol"')
        jstr = json.dumps(text)
        js = json.loads(jstr)
        df = pd.DataFrame(pd.read_json(js, dtype={'code':object}), columns=cf.DAY_TRADING_COLUMNS)
        df = df.drop('symbol', axis=1)

        return df
    
    
    def indexETF(self):
        """
        获取指数ETF及其相关数据
        return
        ------
        DataFrame or List: [{'fund_id':, 'fund_nm':, ...}, ...]
            fund_id:             基金代码
            fund_nm:             基金名称
            index_id:            跟踪指数代码
            creation_unit:       最小申赎单位（万份）
            amount:              份额（万份）
            unit_total:          规模（亿元）
            unit_incr:           规模变化（亿元）
            price:               现价
            volume:              成交额（万元）
            increase_rt:         涨幅（%）
            estimate_value:      估值
            discount_rt:         溢价率（%）
            fund_nav:            净值
            nav_dt:              净值日期
            index_nm:            指数名称
            index_increase_rt:   指数涨幅（%）
            pe:                  市盈率
            pb:                  市净率
        """
        self._data = pd.DataFrame()
        
        page = 1
        while(True):
            try:
                request = self._session.get( cf.INDEX_ETF_URL % page )
                text = request.text.replace('%', '')
                dataDict = json.loads(text)
                if dataDict['page'] < page:
                    break
                
                dataList = []
                for row in dataDict['rows']:
                    dataList.append(row['cell'])            
                self._data = self._data.append(pd.DataFrame(dataList, columns = cf.INDEX_ETF_COLS), ignore_index=True)
                
                page += 1
            except Exception as e:
                print(str(e))
        
        self._data[self._data=="-"] = np.NaN
        for col in ['creation_unit', 'amount', 'unit_total', 'unit_incr', 'price', 'volume', 'increase_rt',
                    'estimate_value', 'discount_rt', 'fund_nav', 'index_increase_rt', 'pe', 'pb']:
            self._data[col] = self._data[col].astype(float)

        self._data['index_id'] = self._data['index_id'].map(lambda x : x if x != "0" else None)        
            
        return self._result()
    
    



# -*- coding:utf-8 -*-
"""
投资参考类
Created on 2019/01/03
@author: TabQ
@group : gugu
@contact: 16621596@qq.com
"""

from __future__ import division

import math
import time
import pandas as pd
from pandas.compat import StringIO
import lxml.html
from lxml import etree
import re
import json
from gugu.utility import Utility
from gugu.base import Base, cf

class Reference(Base):
    def distriPlan(self, year=2015, top=25, retry=3, pause=0.001):
        """
        获取分配预案数据
        Parameters
        --------
        year:年份
        top:取最新n条数据，默认取最近公布的25条
        retry : int, 默认 3
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0.001
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
        
        returns
        -------
        DataFrame or List: [{'code', 'name', ...}, ...]
        code:股票代码
        name:股票名称
        year:分配年份
        report_date:公布日期
        divi:分红金额（每10股）
        shares:转增和送股数（每10股）
        """
        self._data = pd.DataFrame()
        
        if top == 'all':
            self._writeHead()
            
            self._data, pages = self.__handleDistriPlan(year, 0, retry, pause)
            for i in range(1, int(pages)):
                self._data = self._data.append(self.__handleDistriPlan(year, i, retry, pause), ignore_index=True)
                
            return self._result()
        elif top <= 25:
            self._data, pages = self.__handleDistriPlan(year, 0, retry, pause)
            self._data = self._data.head(top)
            
            return self._result()
        else:
            if isinstance(top, int):
                self._writeHead()
                
                allPages = int(math.ceil(top/25))
                self._data, pages = self.__handleDistriPlan(year, 0, retry, pause)
                pages = min(allPages, int(pages))
                for i in range(1, pages):
                    self._data = self._data.append(self.__handleDistriPlan(year, i, retry, pause), ignore_index=True)
                
                self._data = self._data.head(top)
                
                return self._result()
            else:
                print(cf.TOP_PARAS_MSG)
    
    
    def __handleDistriPlan(self, year, pageNo, retry, pause):
        for _ in range(retry):
            time.sleep(pause)
            
            try:
                if pageNo > 0:
                    self._writeConsole()
                    
                # http://quotes.money.163.com/data/caibao/fpyg.html?reportdate=2018&sort=declaredate&order=desc&page=0
                html = lxml.html.parse(cf.DP_163_URL % (year, pageNo))  
                res = html.xpath('//table[@class=\"fn_cm_table\"]/tr')
                if self._PY3:
                    sarr = [etree.tostring(node).decode('utf-8') for node in res]
                else:
                    sarr = [etree.tostring(node) for node in res]
                sarr = ''.join(sarr)
                sarr = '<table>%s</table>' % sarr
                df = pd.read_html(sarr)[0]
                df = df.drop(0, axis=1)
                df.columns = cf.DP_163_COLS
                df['divi'] = df['plan'].map(self.__bonus)
                df['shares'] = df['plan'].map(self.__gift)
                df = df.drop('plan', axis=1)
                df['code'] = df['code'].astype(object)
                df['code'] = df['code'].map(lambda x : str(x).zfill(6))
                pages = []
                if pageNo == 0:
                    page = html.xpath('//div[@class=\"mod_pages\"]/a')
                    if len(page)>1:
                        asr = page[len(page)-2]
                        pages = asr.xpath('text()')
            except Exception as e:
                print(e)
            else:
                if pageNo == 0:
                    return df, pages[0] if len(pages)>0 else 0
                else:
                    return df
                
        raise IOError(cf.NETWORK_URL_ERROR_MSG)
    
    
    def __bonus(self, x):
        if self._PY3:
            reg = re.compile(r'分红(.*?)元', re.UNICODE)
            res = reg.findall(x)
            return 0 if len(res)<1 else float(res[0]) 
        else:
            if isinstance(x, unicode):
                s1 = unicode('分红','utf-8')
                s2 = unicode('元','utf-8')
                reg = re.compile(r'%s(.*?)%s'%(s1, s2), re.UNICODE)
                res = reg.findall(x)
                return 0 if len(res)<1 else float(res[0])
            else:
                return 0
            
            
    def __gift(self, x):
        if self._PY3:
            reg1 = re.compile(r'转增(.*?)股', re.UNICODE)
            reg2 = re.compile(r'送股(.*?)股', re.UNICODE)
            res1 = reg1.findall(x)
            res2 = reg2.findall(x)
            res1 = 0 if len(res1)<1 else float(res1[0])
            res2 = 0 if len(res2)<1 else float(res2[0])
            
            return res1 + res2
        else:
            if isinstance(x, unicode):
                s1 = unicode('转增','utf-8')
                s2 = unicode('送股','utf-8')
                s3 = unicode('股','utf-8')
                reg1 = re.compile(r'%s(.*?)%s'%(s1, s3), re.UNICODE)
                reg2 = re.compile(r'%s(.*?)%s'%(s2, s3), re.UNICODE)
                res1 = reg1.findall(x)
                res2 = reg2.findall(x)
                res1 = 0 if len(res1)<1 else float(res1[0])
                res2 = 0 if len(res2)<1 else float(res2[0])
                
                return res1 + res2
            else:
                return 0
            
            
    def forecast(self, year, quarter, retry=3, pause=0.001):
        """
        获取业绩预告数据
        Parameters
        --------
        year:int 年度 e.g:2014
        quarter:int 季度 :1、2、3、4，只能输入这4个季度
           说明：由于是从网站获取的数据，需要一页页抓取，速度取决于您当前网络速度
        retry : int, 默认 3
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0.001
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题   
        Return
        --------
        DataFrame or List: [{'code':, 'name':, ...}, ...]
            code,代码
            name,名称
            type,业绩变动类型【预增、预亏等】
            report_date,发布日期
            pre_eps,上年同期每股收益
            range,业绩变动范围
        """
        self._data = pd.DataFrame()
        
        if Utility.checkQuarter(year, quarter) is True:
            self._writeHead()
            self._data =  self.__handleForecast(year, quarter, 1, pd.DataFrame(), retry, pause)
            self._data = pd.DataFrame(self._data, columns=cf.FORECAST_COLS)
            self._data['code'] = self._data['code'].map(lambda x: str(x).zfill(6))
            
            return self._result()
        
        
    def __handleForecast(self, year, quarter, pageNo, dataArr, retry, pause):
        self._writeConsole()
        
        for _ in range(retry):
            time.sleep(pause)
            
            try:
                # http://vip.stock.finance.sina.com.cn/q/go.php/vFinanceAnalyze/kind/performance/index.phtml?s_i=&s_a=&s_c=&s_type=&reportdate=2018&quarter=3&p=1&num=60
                request = self._session.get( cf.FORECAST_URL%( year, quarter, pageNo, cf.PAGE_NUM[1]), timeout=10 )
                request.encoding = 'gbk'
                text = request.text.replace('--', '')
                html = lxml.html.parse(StringIO(text))
                res = html.xpath("//table[@class=\"list_table\"]/tr")
                if self._PY3:
                    sarr = [etree.tostring(node).decode('utf-8') for node in res]
                else:
                    sarr = [etree.tostring(node) for node in res]
                sarr = ''.join(sarr)
                sarr = '<table>%s</table>'%sarr
                df = pd.read_html(sarr)[0]
                df = df.drop([4, 5, 8], axis=1)
                df.columns = cf.FORECAST_COLS
                dataArr = dataArr.append(df, ignore_index=True)
                nextPage = html.xpath('//div[@class=\"pages\"]/a[last()]/@onclick')
                if len(nextPage)>0:
                    pageNo = re.findall(r'\d+',nextPage[0])[0]
                    return self.__handleForecast(year, quarter, pageNo, dataArr, retry, pause)
                else:
                    return dataArr
            except Exception as e:
                    print(e)
                    
        raise IOError(cf.NETWORK_URL_ERROR_MSG)
    
    
    def restrictedLift(self, year=None, month=None, retry=3, pause=0.001):
        """
        获取限售股解禁数据
        Parameters
        --------
        year:年份,默认为当前年
        month:解禁月份，默认为当前月
        retry : int, 默认 3
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
        
        Return
        ------
        DataFrame or List: [{'code':, 'name':, ...}, ...]
        code:股票代码
        name:名称
        date:解禁日期
        count:解禁数量（万股）
        ratio:占总盘比率
        """
        self._data = pd.DataFrame()
        
        year = Utility.getYear() if year is None else year
        month = Utility.getMonth() if month is None else month
        
        for _ in range(retry):
            time.sleep(pause)
            
            try:
                # http://datainterface.eastmoney.com/EM_DataCenter/JS.aspx?type=FD&sty=BST&st=3&sr=true&fd=2019&stat=1
                request = self._session.get( cf.RL_URL % (year, month), timeout = 10 )
                if self._PY3:
                    request.encoding = 'utf-8'
                lines = request.text
            except Exception as e:
                print(e)
            else:
                da = lines[3:len(lines)-3]
                list =  []
                for row in da.split('","'):
                    list.append([data for data in row.split(',')])
                self._data = pd.DataFrame(list)
                self._data = self._data[[1, 3, 4, 5, 6]]
                for col in [5, 6]:
                    self._data[col] = self._data[col].astype(float)
                self._data[5] = self._data[5]/10000
                self._data[6] = self._data[6]*100
                self._data[5] = self._data[5].map(cf.FORMAT)
                self._data[6] = self._data[6].map(cf.FORMAT)
                self._data.columns = cf.RL_COLS
                
                return self._result()
            
        raise IOError(cf.NETWORK_URL_ERROR_MSG)


    def fundHoldings(self, year, quarter, retry=3, pause=0.001):
        """
        获取基金持股数据
        Parameters
        --------
        year:年份e.g 2014
        quarter:季度（只能输入1，2，3，4这个四个数字）
        retry : int, 默认 3
                    如遇网络等问题重复执行的次数 
        pause : int, 默认 0
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
        
        Return
        ------
        DataFrame or List: [{'code':, 'name':, ...}, ...]
        code:股票代码
        name:名称
        date:报告日期
        nums:基金家数
        nlast:与上期相比（增加或减少了）
        count:基金持股数（万股）
        clast:与上期相比
        amount:基金持股市值
        ratio:占流通盘比率
        """
        self._data = pd.DataFrame()
        
        start, end = cf.QUARTS_DIC[str(quarter)]
        if quarter == 1:
            start = start % str(year-1)
            end = end % year
        else:
            start, end = start % year, end % year

        self._writeHead()

        self._data, pages = self.__handleFoundHoldings(start, end, 0, retry, pause)
        for idx in range(1, pages):
            self._data = self._data.append(self.__handleFoundHoldings(start, end, idx, retry, pause), ignore_index=True)

        return self._result()


    def __handleFoundHoldings(self, start, end, pageNo, retry, pause):
        for _ in range(retry):
            time.sleep(pause)
            if pageNo>0:
                    self._writeConsole()
            try:
                # http://quotes.money.163.com/hs/marketdata/service/jjcgph.php?host=/hs/marketdata/service/jjcgph.php&page=0&query=start:2018-06-30;end:2018-09-30&order=desc&count=60&type=query&req=73259
                request = self._session.get( cf.FUND_HOLDS_URL % (pageNo, start, end, Utility.random(5)), timeout=10 )
                if self._PY3:
                    request.encoding = 'utf-8'
                lines = request.text
                lines = lines.replace('--', '0')
                lines = json.loads(lines)
                data = lines['list']
                df = pd.DataFrame(data)
                df = df.drop(['CODE', 'ESYMBOL', 'EXCHANGE', 'NAME', 'RN', 'SHANGQIGUSHU', 'SHANGQISHIZHI', 'SHANGQISHULIANG'], axis=1)
                for col in ['GUSHU', 'GUSHUBIJIAO', 'SHIZHI', 'SCSTC27']:
                    df[col] = df[col].astype(float)
                df['SCSTC27'] = df['SCSTC27']*100
                df['GUSHU'] = df['GUSHU']/10000
                df['GUSHUBIJIAO'] = df['GUSHUBIJIAO']/10000
                df['SHIZHI'] = df['SHIZHI']/10000
                df['GUSHU'] = df['GUSHU'].map(cf.FORMAT)
                df['GUSHUBIJIAO'] = df['GUSHUBIJIAO'].map(cf.FORMAT)
                df['SHIZHI'] = df['SHIZHI'].map(cf.FORMAT)
                df['SCSTC27'] = df['SCSTC27'].map(cf.FORMAT)
                df.columns = cf.FUND_HOLDS_COLS
                df = df[['code', 'name', 'date', 'nums', 'nlast', 'count', 
                            'clast', 'amount', 'ratio']]
            except Exception as e:
                print(e)
            else:
                if pageNo == 0:
                    return df, int(lines['pagecount'])
                else:
                    return df

        raise IOError(cf.NETWORK_URL_ERROR_MSG)


    def ipo(self, retry=3, pause=0.001):
        """
        获取新股上市数据
        Parameters
        --------
        retry : int, 默认 3
                    如遇网络等问题重复执行的次数 
        pause : int, 默认 0
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
        
        Return
        ------
        DataFrame or List: [{'code':, 'name':, ...}, ...]
        code:股票代码
        xcode:申购代码
        name:名称
        ipo_date:上网发行日期
        issue_date:上市日期
        amount:发行数量(万股)
        markets:上网发行数量(万股)
        price:发行价格(元)
        pe:发行市盈率
        limit:个人申购上限(万股)
        funds：募集资金(亿元)
        ballot:网上中签率(%)
        """
        self._data = pd.DataFrame()

        self._writeHead()

        self._data = self.__handleIpo(self._data, 1, retry, pause)

        return self._result()


    def __handleIpo(self, data, pageNo, retry, pause):
        self._writeConsole()

        for _ in range(retry):
            time.sleep(pause)

            try:
                # http://vip.stock.finance.sina.com.cn/corp/view/vRPD_NewStockIssue.php?page=1&cngem=0&orderBy=NetDate&orderType=desc
                html = lxml.html.parse(cf.NEW_STOCKS_URL % pageNo)
                res = html.xpath('//table[@id=\"NewStockTable\"]/tr')
                if not res:
                    return data
                
                if self._PY3:
                    sarr = [etree.tostring(node).decode('utf-8') for node in res]
                else:
                    sarr = [etree.tostring(node) for node in res]
                sarr = ''.join(sarr)
                sarr = sarr.replace('<font color="red">*</font>', '')
                sarr = '<table>%s</table>'%sarr
                df = pd.read_html(StringIO(sarr), skiprows=[0, 1])[0]
                df = df.drop([df.columns[idx] for idx in [12, 13, 14, 15]], axis=1)
                df.columns = cf.NEW_STOCKS_COLS
                df['code'] = df['code'].map(lambda x : str(x).zfill(6))
                df['xcode'] = df['xcode'].map(lambda x : str(x).zfill(6))
                res = html.xpath('//table[@class=\"table2\"]/tr[1]/td[1]/a/text()')
                tag = '下一页' if self._PY3 else unicode('下一页', 'utf-8')
                hasNext = True if tag in res else False 
                data = data.append(df, ignore_index=True)
                pageNo += 1
                if hasNext:
                    data = self.__handleIpo(data, pageNo, retry, pause)
            except Exception as ex:
                print(ex)
            else:
                return data
            
            
    def shMargins(self, retry=3, pause=0.001):
        """
        沪市融资融券历史数据
        Parameters
        --------
        retry : int, 默认 3
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
        
        Return
        ------
        DataFrame or List: [{'date':, 'close':, ...}, ...]
            date: 日期
            close: 上证指数收盘点数
            zdf: 上证指数收盘涨跌幅(%)
            rzye: 融资余额(元)
            rzyezb: 融资余额占比(%)
            rzmre: 融资买入额(元)
            rzche: 融资偿还额(元)
            rzjmre: 融资净买入额(元)
            rqye: 融券余额(元)
            rqyl: 融券余量(股)
            rqmcl: 融券卖出量(股)
            rqchl: 融券偿还量(股)
            rqjmcl: 融券净卖出量(股)
            rzrqye: 融资融券余额(元)
            rzrqyecz: 融资融券余额差值(元)
        """
        self._data = pd.DataFrame()
        
        self._writeHead()
        
        self._data = self.__handleMargins(self._data, 1, 'SH', Utility.random(8), cf.MAR_COLS, retry, pause)
        self._data.rename(columns={'tdate':'date'}, inplace=True)
        
        return self._result()
    
    
    def szMargins(self, retry=3, pause=0.001):
        """
        深市融资融券历史数据
        Parameters
        --------
        retry : int, 默认 3
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
        
        Return
        ------
        DataFrame or List: [{'date':, 'close':, ...}, ...]
            date: 日期
            close: 深证成指收盘点数
            zdf: 深证成指收盘涨跌幅(%)
            rzye: 融资余额(元)
            rzyezb: 融资余额占比(%)
            rzmre: 融资买入额(元)
            rzche: 融资偿还额(元)
            rzjmre: 融资净买入额(元)
            rqye: 融券余额(元)
            rqyl: 融券余量(股)
            rqmcl: 融券卖出量(股)
            rqchl: 融券偿还量(股)
            rqjmcl: 融券净卖出量(股)
            rzrqye: 融资融券余额(元)
            rzrqyecz: 融资融券余额差值(元)
        """
        self._data = pd.DataFrame()
        
        self._writeHead()
        
        self._data = self.__handleMargins(self._data, 1, 'SZ', Utility.random(8), cf.MAR_COLS, retry, pause)
        self._data.rename(columns={'tdate':'date'}, inplace=True)
        
        return self._result()
    
    
    def __handleMargins(self, dataArr, page, market, randInt, column, retry, pause):
        self._writeConsole()
        
        for _ in range(retry):
            time.sleep(pause)
            
            try:
                request = self._session.get( cf.MAR_URL % (page, market, randInt) )
                text = request.text.split('=')[1]
                text = text.replace('{pages:', '{"pages":').replace(',data:', ',"data":').replace('T00:00:00', '').replace('"-"', '0')
                dataDict = Utility.str2Dict(text)
                data = dataDict['data']
                df = pd.DataFrame(data, columns=column)
                
                df['close'] = df['close'].map(cf.FORMAT)
                df['rzyezb'] = df['rzyezb'].astype(float)
                
                dataArr = dataArr.append(df, ignore_index=True)
                if page < dataDict['pages']:
                    dataArr = self.__handleMargins(dataArr, page+1, market, randInt, column, retry, pause)
            except Exception as e:
                print(e)
            else:
                return dataArr
                
        raise IOError(cf.NETWORK_URL_ERROR_MSG)
    
    
    def marginDetailsAllByDate(self, date, retry=3, pause=0.001):
        """
        按日期获取两市融资融券明细列表
        Parameters
        --------
        date  : string
                     选择日期 format：YYYY-MM-DD
        retry : int, 默认 3
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
        
        Return
        ------
        DataFrame or List: [{'code':, 'name':, ...}, ...]
            code: 股票代码
            name: 名称
            rzye: 当日融资余额(元)
            rzyezb: 当日融资余额占比(%)
            rzmre: 当日融资买入额(元)
            rzche: 当日融资偿还额(元)
            rzjmre: 当日融资净买入额(元)
            rqye: 当日融券余额(元)
            rqyl: 当日融券余量(股)
            rqmcl: 当日融券卖出量(股)
            rqchl: 当日融券偿还量(股)
            rqjmcl: 当日融券净卖出量(股)
            rzrqye: 当日融资融券余额(元)
            rzrqyecz: 当日融资融券余额差值(元)
        """
        self._data = pd.DataFrame()
        
        self._writeHead()
        
        self._data = self.__handleMarginDetailsAllByDate(self._data, date, 1, Utility.random(8), retry, pause)
        self._data.rename(columns={'scode':'code', 'sname':'name'}, inplace=True)
        
        return self._result()
    
    
    def __handleMarginDetailsAllByDate(self, dataArr, date, page, randInt, retry, pause):
        self._writeConsole()
        
        for _ in range(retry):
            time.sleep(pause)
            
            try:
                request = self._session.get(cf.MAR_BOTH_DETAIL % (date, page, randInt))
                text = request.text.split('=')[1]
                text = text.replace('{pages:', '{"pages":').replace(',data:', ',"data":').replace('"-"', '0')
                dataDict = Utility.str2Dict(text)
                data = dataDict['data']
                df = pd.DataFrame(data, columns=cf.MAR_DET_All_COLS)
                
                df['date'] = date
                df['rzyezb'] = df['rzyezb'].astype(float)
                    
                dataArr = dataArr.append(df, ignore_index=True)
                if page < dataDict['pages']:
                    dataArr = self.__handleMarginDetailsAllByDate(dataArr, date, page+1, randInt, retry, pause)
            except Exception as e:
                print(e)
            else:
                return dataArr
            
        raise IOError(cf.NETWORK_URL_ERROR_MSG)
    
    
    def marginTotal(self, retry=3, pause=0.001):
        """
        两市合计融资融券历史数据
        Parameters
        --------
        retry : int, 默认 3
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
        
        Return
        ------
        DataFrame or List: [{'date':, 'close':, ...}, ...]
            date: 日期
            close: 沪深300收盘点数
            zdf: 沪深300收盘涨跌幅(%)
            rzye: 融资余额(元)
            rzyezb: 融资余额占比(%)
            rzmre: 融资买入额(元)
            rzche: 融资偿还额(元)
            rzjmre: 融资净买入额(元)
            rqye: 融券余额(元)
            rqyl: 融券余量(股)
            rqmcl: 融券卖出量(股)
            rqchl: 融券偿还量(股)
            rqjmcl: 融券净卖出量(股)
            rzrqye: 融资融券余额(元)
            rzrqyecz: 融资融券余额差值(元)
        """
        self._data = pd.DataFrame()
        
        self._writeHead()
        
        self._data = self.__handleMarginTotal(self._data, 1, Utility.random(8), retry, pause)
        self._data.rename(columns={'tdate':'date'}, inplace=True)
        
        return self._result()
    
    
    def __handleMarginTotal(self, dataArr, page, randInt, retry, pause):
        self._writeConsole()
        
        for _ in range(retry):
            time.sleep(pause)
            
            try:
                request = self._session.get(cf.MAR_TOTAL_URL % (page, randInt), timeout=10)
                text = request.text.split('=')[1]
                text = text.replace('{pages:', '{"pages":').replace(',data:', ',"data":').replace('T00:00:00', '').replace('"-"', '0')
                dataDict = Utility.str2Dict(text)
                data = dataDict['data']
                df = pd.DataFrame(data, columns=cf.MAR_TOTAL_COLS)
                
                df['close'] = df['close'].map(cf.FORMAT)
                df['rzyezb'] = df['rzyezb'].astype(float)
                
                dataArr = dataArr.append(df, ignore_index=True)
                if page < dataDict['pages']:
                    dataArr = self.__handleMarginTotal(dataArr, page+1, randInt, retry, pause)
            except Exception as e:
                print(e)
            else:
                return dataArr
            
        raise IOError(cf.NETWORK_URL_ERROR_MSG)
    



# -*- coding:utf-8 -*-
"""
股票信息类
Created on 2019/01/02
@author: TabQ
@group : gugu
@contact: 16621596@qq.com
"""
from __future__ import division

import time
import pandas as pd
from pandas.compat import StringIO
import lxml.html
from lxml import etree
import re
from gugu.utility import Utility
from gugu.base import Base, cf

class StockInfo(Base):
    def stockProfiles(self, retry=3, pause=0.001):
        """
        获取上市公司基本情况
        Parameters
        --------
        retry : int, 默认 3
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
                    
        Return
        --------
        DataFrame or List: [{'code':, 'name':, ...}, ...]
                   code,代码
                   name,名称
                   city,所在城市
                   staff,员工人数
                   date,上市日期
                   industry,行业分类
                   pro_type,产品类型
                   main,主营业务
        """
        self._data = pd.DataFrame()
        
        self._writeHead()
        
        date = '%s-12-31' % Utility.getYear()
        self._data = pd.DataFrame()
        
        self._data = self.__handleStockProfiles(self._data, date, 1, retry, pause)
        
        return self._result()
    
    
    def __handleStockProfiles(self, dataArr, date, page, retry, pause):
        self._writeConsole()
        
        for _ in range(retry):
            time.sleep(pause)
            
            try:
                html = lxml.html.parse(cf.ALL_STOCK_PROFILES_URL % (date, page))
                res = html.xpath('//table[@id="myTable04"]/tbody/tr')
                if not res:
                    return dataArr
                
                if self._PY3:
                    sarr = [etree.tostring(node).decode('utf-8') for node in res]
                else:
                    sarr = [etree.tostring(node) for node in res]
                sarr = ''.join(sarr)
                sarr = '<table>%s</table>' % sarr
                
                df = pd.read_html(sarr)[0]
                df = df.drop([0, 3, 5, 6, 7, 10, 11], axis = 1)
                df.columns = cf.ALL_STOCK_PROFILES_COLS
                df['code'] = df['code'].map(lambda x: str(x).zfill(6))
                
                dataArr = dataArr.append(df, ignore_index=True)
            except Exception as e:
                print(e)
            else:
                return self.__handleStockProfiles(dataArr, date, page+1, retry, pause)
        
    
    def report(self, year, quarter, retry=3, pause=0.001):
        """
        获取业绩报表数据
        Parameters
        --------
        year:int 年度 e.g:2014
        quarter:int 季度 :1、2、3、4，只能输入这4个季度
           说明：由于是从网站获取的数据，需要一页页抓取，速度取决于您当前网络速度
        retry : int, 默认 3
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0.001
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题   
        Return
        --------
        DataFrame or List: [{'code':, 'name':, ...}, ...]
            code,代码
            name,名称
            eps,每股收益
            eps_yoy,每股收益同比(%)
            bvps,每股净资产
            roe,净资产收益率(%)
            epcf,每股现金流量(元)
            net_profits,净利润(万元)
            profits_yoy,净利润同比(%)
            distrib,分配方案
            report_date,发布日期
        """
        self._data = pd.DataFrame()
        
        if Utility.checkQuarter(year, quarter) is True:
            self._writeHead()
            
            # http://vip.stock.finance.sina.com.cn/q/go.php/vFinanceAnalyze/kind/mainindex/index.phtml?s_i=&s_a=&s_c=&reportdate=2018&quarter=3&p=1&num=60
            self._data = self.__parsePage(cf.REPORT_URL, year, quarter, 1, cf.REPORT_COLS, pd.DataFrame(), retry, pause, 11)
            if self._data is not None:
                self._data['code'] = self._data['code'].map(lambda x:str(x).zfill(6))
                
            return self._result()
        
    def profit(self, year, quarter, retry=3, pause=0.001):
        """
        获取盈利能力数据
        Parameters
        --------
        year:int 年度 e.g:2014
        quarter:int 季度 :1、2、3、4，只能输入这4个季度
           说明：由于是从网站获取的数据，需要一页页抓取，速度取决于您当前网络速度
        retry : int, 默认 3
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0.001
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题   
        Return
        --------
        DataFrame or List: [{'code':, 'name':, ...}, ...]
            code,代码
            name,名称
            roe,净资产收益率(%)
            net_profit_ratio,净利率(%)
            gross_profit_rate,毛利率(%)
            net_profits,净利润(万元)
            eps,每股收益
            business_income,营业收入(百万元)
            bips,每股主营业务收入(元)
        """
        self._data = pd.DataFrame()
        
        if Utility.checkQuarter(year, quarter) is True:
            self._writeHead()
            
            # http://vip.stock.finance.sina.com.cn/q/go.php/vFinanceAnalyze/kind/profit/index.phtml?s_i=&s_a=&s_c=&reportdate=2018&quarter=3&p=1&num=60
            self._data = self.__parsePage(cf.PROFIT_URL, year, quarter, 1, cf.PROFIT_COLS, pd.DataFrame(), retry, pause)
            if self._data is not None:
                self._data['code'] = self._data['code'].map(lambda x: str(x).zfill(6))
                
            return self._result()
        
    def operation(self, year, quarter, retry=3, pause=0.001):
        """
        获取营运能力数据
        Parameters
        --------
        year:int 年度 e.g:2014
        quarter:int 季度 :1、2、3、4，只能输入这4个季度
           说明：由于是从网站获取的数据，需要一页页抓取，速度取决于您当前网络速度
        retry : int, 默认 3
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0.001
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题   
        Return
        --------
        DataFrame or List: [{'code':, 'name':, ...}, ...]
            code,代码
            name,名称
            arturnover,应收账款周转率(次)
            arturndays,应收账款周转天数(天)
            inventory_turnover,存货周转率(次)
            inventory_days,存货周转天数(天)
            currentasset_turnover,流动资产周转率(次)
            currentasset_days,流动资产周转天数(天)
        """
        self._data = pd.DataFrame()
        
        if Utility.checkQuarter(year, quarter) is True:
            self._writeHead()
            
            # http://vip.stock.finance.sina.com.cn/q/go.php/vFinanceAnalyze/kind/operation/index.phtml?s_i=&s_a=&s_c=&reportdate=2018&quarter=3&p=1&num=60
            self._data = self.__parsePage(cf.OPERATION_URL, year, quarter, 1, cf.OPERATION_COLS, pd.DataFrame(), retry, pause)
            if self._data is not None:
                self._data['code'] = self._data['code'].map(lambda x: str(x).zfill(6))
                
            return self._result()
        
    def growth(self, year, quarter, retry=3, pause=0.001):
        """
        获取成长能力数据
        Parameters
        --------
        year:int 年度 e.g:2014
        quarter:int 季度 :1、2、3、4，只能输入这4个季度
           说明：由于是从网站获取的数据，需要一页页抓取，速度取决于您当前网络速度
        retry : int, 默认 3
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0.001
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题   
        Return
        --------
        DataFrame or List: [{'code':, 'name':, ...}, ...]
            code,代码
            name,名称
            mbrg,主营业务收入增长率(%)
            nprg,净利润增长率(%)
            nav,净资产增长率
            targ,总资产增长率
            epsg,每股收益增长率
            seg,股东权益增长率
        """
        self._data = pd.DataFrame()
        
        if Utility.checkQuarter(year, quarter) is True:
            self._writeHead()
            
            # http://vip.stock.finance.sina.com.cn/q/go.php/vFinanceAnalyze/kind/grow/index.phtml?s_i=&s_a=&s_c=&reportdate=2018&quarter=3&p=1&num=60
            self._data = self.__parsePage(cf.GROWTH_URL, year, quarter, 1, cf.GROWTH_COLS, pd.DataFrame(), retry, pause)
            if self._data is not None:
                self._data['code'] = self._data['code'].map(lambda x: str(x).zfill(6))
                 
            return self._result()
        
    def debtPaying(self, year, quarter, retry=3, pause=0.001):
        """
        获取偿债能力数据
        Parameters
        --------
        year:int 年度 e.g:2014
        quarter:int 季度 :1、2、3、4，只能输入这4个季度
           说明：由于是从网站获取的数据，需要一页页抓取，速度取决于您当前网络速度
        retry : int, 默认 3
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0.001
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题   
        Return
        --------
        DataFrame or List: [{'code':, 'name':, ...}, ...]
            code,代码
            name,名称
            currentratio,流动比率
            quickratio,速动比率
            cashratio,现金比率
            icratio,利息支付倍数
            sheqratio,股东权益比率
            adratio,股东权益增长率
        """
        self._data = pd.DataFrame()
        
        if Utility.checkQuarter(year, quarter) is True:
            self._writeHead()
            
            # http://vip.stock.finance.sina.com.cn/q/go.php/vFinanceAnalyze/kind/debtpaying/index.phtml?s_i=&s_a=&s_c=&reportdate=2018&quarter=3&p=1&num=60
            self._data = self.__parsePage(cf.DEBTPAYING_URL, year, quarter, 1, cf.DEBTPAYING_COLS, pd.DataFrame(), retry, pause)
            if self._data is not None:
                self._data['code'] = self._data['code'].map(lambda x: str(x).zfill(6))
                
            return self._result()
        
    def cashFlow(self, year, quarter, retry=3, pause=0.001):
        """
        获取现金流量数据
        Parameters
        --------
        year:int 年度 e.g:2014
        quarter:int 季度 :1、2、3、4，只能输入这4个季度
           说明：由于是从网站获取的数据，需要一页页抓取，速度取决于您当前网络速度
        retry : int, 默认 3
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0.001
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题   
        Return
        --------
        DataFrame or List: [{'code':, 'name':, ...}, ...]
            code,代码
            name,名称
            cf_sales,经营现金净流量对销售收入比率
            rateofreturn,资产的经营现金流量回报率
            cf_nm,经营现金净流量与净利润的比率
            cf_liabilities,经营现金净流量对负债比率
            cashflowratio,现金流量比率
        """
        self._data = pd.DataFrame()
        
        if Utility.checkQuarter(year, quarter) is True:
            self._writeHead()
            
            # http://vip.stock.finance.sina.com.cn/q/go.php/vFinanceAnalyze/kind/cashflow/index.phtml?s_i=&s_a=&s_c=&reportdate=2018&quarter=3&p=1&num=60
            self._data = self.__parsePage(cf.CASHFLOW_URL, year, quarter, 1, cf.CASHFLOW_COLS, pd.DataFrame(), retry, pause)
            if self._data is not None:
                self._data['code'] = self._data['code'].map(lambda x: str(x).zfill(6))
                
            return self._result()
        
    def __parsePage(self, url, year, quarter, page, column, dataArr, retry, pause, drop_column=None):
        self._writeConsole()
        
        for _ in range(retry):
            time.sleep(pause)
            
            try:
                request = self._session.get( url % (year, quarter, page, cf.PAGE_NUM[1]), timeout=10 )
                request.encoding = 'gbk'
                text = request.text.replace('--', '')
                html = lxml.html.parse(StringIO(text))
                res = html.xpath("//table[@class=\"list_table\"]/tr")
                if self._PY3:
                    sarr = [etree.tostring(node).decode('utf-8') for node in res]
                else:
                    sarr = [etree.tostring(node) for node in res]
                sarr = ''.join(sarr)
                sarr = '<table>%s</table>'%sarr
                df = pd.read_html(sarr)[0]
                if drop_column is not None:
                    df = df.drop(drop_column, axis=1)
                df.columns = column
                dataArr = dataArr.append(df, ignore_index=True)
                nextPage = html.xpath('//div[@class=\"pages\"]/a[last()]/@onclick')
                if len(nextPage) > 0:
                    page = re.findall(r'\d+', nextPage[0])[0]
                    return self.__parsePage(url, year, quarter, page, column, dataArr, retry, pause, drop_column)
                else:
                    return dataArr
            except Exception as e:
                print(e)
                
        raise IOError(cf.NETWORK_URL_ERROR_MSG)
    
    



# -*- coding:utf-8 -*-
"""
股票所属类
Created on 2019/01/10
@author: TabQ
@group : gugu
@contact: 16621596@qq.com
"""
import time
import json
import re
import pandas as pd
from gugu.base import Base, cf

class Classify(Base):        
    def byIndustry(self, std='sina', retry=3, pause=0.001):
        """
        获取行业分类数据
        Parameters
        ----------
        std : string
                sina:新浪行业 sw：申万行业
        retry : int, 默认 10
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0.001
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
        
        Returns
        -------
        DataFrame or List: [{'code':, 'name':, ...}, ...]
            code :股票代码
            name :股票名称
            c_name :行业名称
        """
        self._data = pd.DataFrame()
        
        if std == 'sw':
            # http://vip.stock.finance.sina.com.cn/q/view/SwHy.php
            df = self.__getTypeData(cf.SINA_INDUSTRY_INDEX_URL % 'SwHy.php')
        else:
            # http://vip.stock.finance.sina.com.cn/q/view/newSinaHy.php
            df = self.__getTypeData(cf.SINA_INDUSTRY_INDEX_URL % 'newSinaHy.php')
            
        self._writeHead()
        for row in df.values:
            rowDf =  self.__getDetail(row[0], retry, pause)
            rowDf['c_name'] = row[1]
            self._data = self._data.append(rowDf, ignore_index=True)
        
        return self._result()
    
    
    def byConcept(self, retry=3, pause=0.001):
        """
        获取概念分类数据
        Parameters
        ----------
        retry : int, 默认 3
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0.001
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
        Return
        --------
        DataFrame
            code :股票代码
            name :股票名称
            c_name :概念名称
        """
        self._data = pd.DataFrame()
        
        self._writeHead()
        # http://money.finance.sina.com.cn/q/view/newFLJK.php?param=class
        df = self.__getTypeData( cf.SINA_CONCEPTS_INDEX_URL )
        
        for row in df.values:
            rowDf = self.__getDetail(row[0], retry, pause)
            rowDf['c_name'] = row[1]
            self._data = self._data.append(rowDf)
            
        return self._result()
    
            
    def __getTypeData(self, url):
        try:
            request = self._session.get(url, timeout=10)
            request.encoding = 'gbk'
            text = request.text.split('=')[1]
            dataJson = json.loads(text)
            df = pd.DataFrame([[row.split(',')[0], row.split(',')[1]] for row in dataJson.values()], columns=['tag', 'name'])
            
            return df
        except Exception as e:
            print(str(e))
    
    def __getDetail(self, tag, retry=3, pause=0.001):
        self._writeConsole()
        
        for retryCount in range(retry):
            time.sleep(pause)
            
            try:
                # http://vip.stock.finance.sina.com.cn/quotes_service/api/json_v2.php/Market_Center.getHQNodeData?page=1&num=1000&sort=symbol&asc=1&node=new_zhhy&symbol=&_s_r_a=page
                request = self._session.get( cf.SINA_DATA_DETAIL_URL % tag, timeout=10 )
                reg = re.compile(r'\,(.*?)\:')
                text = reg.sub(r',"\1":', request.text)
                text = text.replace('"{symbol', '{"symbol')
                text = text.replace('{symbol', '{"symbol"')
                jstr = json.dumps(text)
                js = json.loads(jstr)
                
                df = pd.DataFrame(pd.read_json(js, dtype={'code':object}), columns=cf.FOR_CLASSIFY_B_COLS)
                
                return df
            except:
                time.sleep((retryCount+1)*10)
            
        raise IOError(cf.NETWORK_ERR_MSG)
    
    



# -*- coding:utf-8 -*-
"""
公用类
Created on 2019/01/15
@author: TabQ
@group : gugu
@contact: 16621596@qq.com
"""

from __future__ import division
import datetime
import json
import requests
from requests.adapters import HTTPAdapter
from urllib3.util import Retry
import gugu.config as cf

class Utility():
    @staticmethod    
    def str2Dict(string):
        string = eval(string, type('Dummy', (dict,), dict(__getitem__ = lambda s, n:n))())
        string = json.dumps(string)
        
        return json.loads(string)
    
    
    @staticmethod
    def fceil(number, ndigits=2):
        if number == 0:
            return number
        
        tail_number = 1 / pow(10, ndigits)
        ndigits_number = round(number, ndigits)
        
        result = number - ndigits_number
        if result == 0:
            return number
        elif result > 0:
            return ndigits_number + tail_number
        else:
            return ndigits_number
    
    
    @staticmethod
    def checkQuarter(year, quarter):
        if isinstance(year, str) or year < 1989 :
            raise TypeError(cf.DATE_CHK_MSG)
        elif quarter is None or isinstance(quarter, str) or quarter not in [1, 2, 3, 4]:
            raise TypeError(cf.DATE_CHK_Q_MSG)
        else:
            return True
        
        
    @staticmethod
    def checkLhbInput(last):
        if last not in [5, 10, 30, 60]:
            raise TypeError(cf.LHB_MSG)
        else:
            return True
        
    
    @staticmethod
    def symbol(code):
        """
                            生成symbol代码标志
        """
        if code in cf.INDEX_LABELS:
            return cf.INDEX_LIST[code]
        elif len(code) != 6 :
            return ''
        else:
            return 'sh%s' % code if code[:1] in ['5', '6', '9'] else 'sz%s' % code 
        
        
    @staticmethod
    def random(n=13):
        from random import randint
        
        start = 10**(n-1)
        end = (10**n)-1
        
        return str(randint(start, end))
    
    
    @staticmethod
    def getToday():
        return str(datetime.datetime.today().date())
    
    
    @staticmethod
    def getHour():
        return datetime.datetime.today().hour
        
    
    @staticmethod
    def getMonth():
        return datetime.datetime.today().month
        
    
    @staticmethod
    def getYear():
        return datetime.datetime.today().year
        
    
    @staticmethod
    def getTodayLastYear():
        return str(Utility.getDateInterval(days=-365))

    
    @staticmethod
    def getTodayLastMonth():
        return str(Utility.getDateInterval(days=-30))
        
    
    @staticmethod
    def getTodayLastWeek():
        return str(Utility.getDateInterval(days=-7))
        
        
    @staticmethod
    def getDateInterval(date=str(datetime.datetime.today().date()), days=-1):
        return datetime.datetime.strptime(date, '%Y-%m-%d').date() + datetime.timedelta(days)
    
    
    @staticmethod
    def diffDays(start=None, end=None):
        d1 = datetime.datetime.strptime(end, '%Y-%m-%d')
        d2 = datetime.datetime.strptime(start, '%Y-%m-%d')
        delta = d1 - d2
        
        return delta.days
    
    
    @staticmethod
    def ttDates(start='', end=''):
        startyear = int(start[0:4])
        endyear = int(end[0:4])
        dates = [d for d in range(startyear, endyear+1, 2)]
        
        return dates
    
    
    @staticmethod
    def lastTradeDate(date=str(datetime.datetime.today().date())):
        lastdate = str(Utility.getDateInterval(date, -1))
        
        if Utility.isTradeDay(lastdate):
            return lastdate
        else:
            return Utility.lastTradeDate(lastdate)
        
        
    @staticmethod
    def nextTradeDate(date=str(datetime.datetime.today().date())):
        nextdate = str(Utility.getDateInterval(date, 1))
        
        if Utility.isTradeDay(nextdate):
            return nextdate
        else:
            return Utility.nextTradeDate(nextdate)
        
        
    @staticmethod
    def isHoliday(date=str(datetime.datetime.today().date())):
        """
                            节假日判断
        Parameters
        ------
            date: string
                                                        查询日期 format：YYYY-MM-DD 为空时取当前日期
        return
        ------
            True or False
        """
        session = requests.Session()
        retry = Retry(connect=5, backoff_factor=1)
        adapter = HTTPAdapter(max_retries=retry)
        session.mount('http://', adapter)
        session.keep_alive = False
        
        try:
            request = session.get( cf.HOLIDAY_URL % date, timeout=10 )
            dataDict = json.loads(request.text)
            if dataDict['code'] != 0:
                raise IOError(cf.HOLIDAY_SERVE_ERR)
            elif dataDict['holiday'] is None:
                return False
            else:
                return True
        except Exception as e:
                print(e)
    
    
    @staticmethod
    def isTradeDay(date=str(datetime.datetime.today().date())):
        """
                            交易日判断
        Parameters
        ------
            date: string
                                                        查询日期 format：YYYY-MM-DD 为空时取当前日期
        return
        ------
            True or False
        """
        date = datetime.datetime.strptime(date, '%Y-%m-%d')
        w = int(date.strftime('%w'))
        
        if w in [6, 0]:
            return False
        else: 
            return not Utility.isHoliday(date)
        



# -*- coding:utf-8 -*-
"""
低风险及套利类
Created on 2019/01/30
@author: TabQ
@group : gugu
@contact: 16621596@qq.com
"""

import re
import json
import pandas as pd
import numpy as np
from gugu.base import Base
import gugu.config as cf
import sys

class LowRiskIntArb(Base):
    def ratingFundA(self):
        """
                            分级基金A及其相关数据
        return
        ------
        DataFrame or List: [{'funda_id':, 'funda_name':, ...}, ...]
            funda_id:                    分级A代码
            funda_name:                  分级A名称
            funda_current_price:         现价
            funda_increase_rt:           涨幅（％）
            funda_volume:                成交额
            funda_value:                 净值
            funda_discount_rt:           折价率（％）
            funda_coupon:                本期利率
            funda_coupon_next:           下期利率
            funda_profit_rt_next:        修正收益率（％）
            funda_index_id:              参考指数代码
            funda_index_name:            参考指数名称
            funda_index_increase_rt:     指数涨幅（％）
            funda_lower_recalc_rt:       下折母基需跌（％）
            lower_recalc_profit_rt:      理论下折收益（％）
            fundb_upper_recalc_rt:       上折母基需涨（％）
            funda_base_est_dis_rt_t1:    T-1溢价率（％）
            funda_base_est_dis_rt_t2:    T-2溢价率（％）
            funda_amount:                A份额（万份）
            funda_amount_increase:       A新增（万份）
            abrate:                      A:B
            next_recalc_dt:              下次定折日期
        """
        self._data = pd.DataFrame()
        
        self._data = self.__parsePage(cf.RATING_FUNDA_URL, cf.RATING_FUNDA_COLS)
        self._data[self._data=="-"] = np.NaN
        self._data['next_recalc_dt'] = self._data['next_recalc_dt'].map(self.__nextRecalcDt)
        for col in ['funda_current_price', 'funda_increase_rt', 'funda_volume', 'funda_value',
                    'funda_discount_rt', 'funda_coupon', 'funda_coupon_next', 'funda_profit_rt_next', 
                    'funda_index_increase_rt', 'funda_lower_recalc_rt', 'lower_recalc_profit_rt', 
                    'fundb_upper_recalc_rt', 'funda_base_est_dis_rt_t1', 'funda_base_est_dis_rt_t2', 
                    'funda_amount_increase']:
            self._data[col] = self._data[col].astype(float)
        
        return self._result()
    
    
    def __nextRecalcDt(self, x):
        pattern = re.compile(r'<span.*?>(\d{4}-\d{2}-\d{2})</span>')
        
        return re.sub(pattern, r'\1', x)
    
    
    def ratingFundB(self):
        """
                            分级基金B及其相关数据
        return
        ------
        DataFrame or List: [{'fundb_id':, 'fundb_name':, ...}, ...]
            fundb_id:                            分级B代码
            fundb_name:                          分级B名称
            fundb_base_fund_id:                  对应母基代码
            funda_id:                            分级A代码
            funda_name:                          分级A名称
            coupon:                              优惠
            manage_fee:                          管理费
            funda_current_price:                 分级A现价
            funda_upper_price:                   分级A最高价
            funda_lower_price:                   分级A最低价
            funda_increase_rt：                  分级A增长率（％）
            fundb_current_price：                分级B现价
            fundb_upper_price：                  分级B最高价
            fundb_lower_price：                  分级B最低价
            fundb_increase_rt：                  分级B增长率（％）
            fundb_volume：                       分级B成交额（万元）
            fundb_value：                        分级B净值
            fundm_value：                        母基净值
            fundb_discount_rt：                  分级B溢价率（％）
            fundb_price_leverage_rt：            分级B价格杠杆
            fundb_net_leverage_rt：              分级B净值杠杆
            fundb_capital_rasising_rt：          分级B融资成本（％）
            fundb_lower_recalc_rt：              下折母基需跌（％）
            fundb_upper_recalc_rt：              上折母基需涨（％）
            b_est_val：                          估值
            fundb_index_id：                     参考指数代码
            fundb_index_name：                   参考指数名称
            fundb_index_increase_rt：            参考指数涨幅（％）
            funda_ratio：                        分级A占比
            fundb_ratio：                        分级B占比
            fundb_base_price：                   分级B基础价格
            fundB_amount：                       B份额（万份）
            fundB_amount_increase：              B新增份额（万份）
            abrate：                             A：B
        """
        self._data = pd.DataFrame()
        
        self._data = self.__parsePage(cf.RATING_FUNDB_URL, cf.RATING_FUNDB_COLS)
        self._data[self._data=="-"] = np.NaN
        for col in ['coupon', 'manage_fee', 'funda_current_price', 'funda_upper_price', 'funda_lower_price',
                    'funda_increase_rt', 'fundb_current_price', 'fundb_upper_price', 'fundb_lower_price',
                    'fundb_increase_rt', 'fundb_volume', 'fundb_value', 'fundm_value', 'fundb_discount_rt',
                    'fundb_price_leverage_rt', 'fundb_net_leverage_rt', 'fundb_capital_rasising_rt', 
                    'fundb_lower_recalc_rt', 'fundb_upper_recalc_rt', 'b_est_val', 'fundb_index_increase_rt', 
                    'fundb_base_price', 'fundB_amount', 'fundB_amount_increase']:
            self._data[col] = self._data[col].astype(float)
            
        return self._result()
    
    
    def ratingFundM(self):
        """
                            分级基金母基金及其相关数据
        return
        ------
        DataFrame or List: [{'base_fund_id':, 'base_fund_nm', ...}, ...]
            base_fund_id:                    母基代码
            base_fund_nm：                   母基名称
            market：                         所属市场
            issue_dt：                       创立日期
            manage_fee：                     管理费
            index_id：                       跟踪指数代码
            index_nm：                       跟踪指数名称
            lower_recalc_price：             下折
            a_ratio：                        A占比
            b_ratio：                        B占比
            next_recalc_dt：                 下次定折
            fundA_id：                       A基代码
            fundA_nm：                       A基名称
            coupon：                         本期利率
            coupon_next：                    下期利率
            fundB_id：                       B基代码
            fundB_nm:                        B基名称
            price：                          母基净值
            base_lower_recalc_rt：           下折需跌（％）
            abrate：                         A：B
        """
        self._data = pd.DataFrame()
        
        self._data = self.__parsePage(cf.RATING_FUNDM_URL, cf.RATING_FUNDM_COLS)
        self._data[self._data=="-"] = np.NaN
        for col in ['manage_fee', 'lower_recalc_price', 'a_ratio', 'b_ratio', 'coupon', 'coupon_next',
                    'price', 'base_lower_recalc_rt']:
            self._data[col] = self._data[col].astype(float)
            
        return self._result()
    
    
    def conBonds(self):
        """
                            可转债及其相关数据
        return
        ------
        DataFrame or List: [{'bond_id':, 'bond_nm':, ...}, ...]
            bond_id:                        代码
            bond_nm:                        名称
            stock_id:                       正股全代码
            stock_nm:                       正股名称
            market:                         所属市场
            convert_price:                  转股价
            convert_dt:                     转股起始日
            issue_dt：                      创立日期
            maturity_dt：                   到期日
            next_put_dt：                   回售起始日
            put_price：                     回售价
            put_count_days：                回售计算天数
            put_total_days：                回售总天数
            redeem_price：                  赎回价
            redeem_price_ratio：            赎回率
            redeem_count_days：             赎回计算天数
            redeem_total_days：             赎回总天数
            orig_iss_amt：                  发行规模（亿）
            curr_iss_amt：                  剩余规模（亿）
            rating_cd：                     债券评级
            issuer_rating_cd：              主体评级
            guarantor：                     担保
            active_fl：                     活跃标志
            ration_rt：                     股东配售率（％）
            pb：                            市净率
            sprice：                        正股价
            sincrease_rt：                  正股涨跌（％）
            last_time：                     最后时间
            convert_value：                 转股价值
            premium_rt：                    溢价率（％）
            year_left：                     剩余年限
            ytm_rt：                        到期税前收益（％）
            ytm_rt_tax：                    到期税后收益（％）
            price：                         现价
            increase_rt：                   涨跌幅（％）
            volume：                        成交额（万元）
            force_redeem_price：            强赎触发价
            put_convert_price：             回售触发价
            convert_amt_ratio：             转债占比
            stock_cd：                      正股简码
            pre_bond_id：                   转债全码
        """
        self._data = pd.DataFrame()
        
        self._data = self.__parsePage(cf.CON_BONDS_URL, cf.CON_BONDS_COLS)
        self._data[self._data=="-"] = np.NaN
        for col in ['convert_price', 'put_price', 'redeem_price', 'redeem_price_ratio', 'orig_iss_amt',
                    'curr_iss_amt', 'ration_rt', 'pb', 'sprice', 'sincrease_rt', 'convert_value', 'premium_rt',
                    'year_left', 'ytm_rt', 'ytm_rt_tax', 'price', 'increase_rt', 'volume', 'force_redeem_price',
                    'put_convert_price', 'convert_amt_ratio']:
            self._data[col] = self._data[col].astype(float)
            
        return self._result()
    
    
    def newConBonds(self):
        """
                            新转债及其相关数据
        return
        ------
        DataFrame or List: [{'bond_id':, 'bond_nm':, ...}, ...]
            bond_id:                    转债代码
            bond_nm:                    转债名称
            apply_cd:                   申购代码
            apply_date:                 申购日期
            list_date:                  上市日期
            amount:                     发行规模（亿元）
            convert_price:              转股价
            pma_rt:                     现价比转股价
            stock_id:                   正股代码
            stock_nm:                   正股名称
            price:                      正股现价
            increase_rt:                正股涨跌幅
            ration_rt:                  股东配售率（％）
            online_amount:              网上规模（亿元）
            lucky_draw_rt:              中签率（％）
            individual_limit:           个人申购限额
            underwriter_rt:             包销比例（％）
            rating_cd:                  评级
            progress_nm:                方案进展
            apply_tips:                 申购提示
            valid_apply:                申购户数（万户）
        """
        self._data = pd.DataFrame()
        
        self._data = self.__parsePage(cf.NEW_CON_BONDS_URL, cf.NEW_CON_BONDS_COLS)
        for col in ['amount', 'convert_price', 'pma_rt', 'price', 'increase_rt', 'ration_rt',
                    'online_amount', 'lucky_draw_rt', 'underwriter_rt', 'valid_apply']:
            self._data[col] = self._data[col].astype(float)
        
        return self._result()
    
    
    def closedStockFund(self):
        """
                            封闭股基及其相关数据
        return
        ------
        DataFrame or List: [{'fund_id':, 'fund_nm':, ...}, ...]
            fund_id:                    代码
            fund_nm:                    名称
            issue_dt:                   创立日期
            duration:                   持续时间
            last_time:                  最后时间
            price:                      现价
            increase_rt：               涨幅（％）
            volume：                    成交金额（万）
            net_value：                 净值
            nav_dt：                    净值日期
            realtime_estimate_value：   最近估值
            discount_rt：               折价率（％）
            left_year：                 剩余年限
            annualize_dscnt_rt：        年化折价率（％）
            quote_incr_rt：             周价增（％）
            nav_incr_rt：               周净增（％）
            spread：                    净价差（％）
            stock_ratio：               股票占比（％）
            report_dt：                 报告日期
            daily_nav_incr_rt：         当日净增（％）
            daily_spread：              日净价差（％）
        """
        self._data = pd.DataFrame()
        
        self._data = self.__parsePage(cf.CLOSED_STOCK_FUND_URL, cf.CLOSED_STOCK_FUND_COLS)
        self._data[self._data=="-"] = np.NaN
        for col in ['price', 'increase_rt', 'volume', 'net_value', 'realtime_estimate_value', 'discount_rt',
                    'left_year', 'annualize_dscnt_rt', 'quote_incr_rt', 'nav_incr_rt', 'spread', 'stock_ratio',
                    'daily_nav_incr_rt', 'daily_spread']:
            self._data[col] = self._data[col].astype(float)
            
        return self._result()
    
    
    def closedBondFund(self):
        """
                            封闭债基及其相关数据
        return
        ------
        DataFrame or List: [{'fund_id':, 'fund_nm':, ...}, ...]
            fund_id:                    代码
            fund_nm:                    名称
            maturity_dt:                到期日期
            left_year:                  剩余年限
            est_val：                   最近估值
            discount_rt：               折价率（％）
            annual_discount_rt：        年化折价率（％）
            trade_price：               现价
            increase_rt：               涨幅（％）
            volume：                    成交金额
            last_time:                  最后时间
            fund_nav：                  最近净值
            last_chg_dt：               净值日期
            price_incr_rt：             净值日增（％）
            stock_ratio：               股票比例
            bond_ratio：                债券比例
            report_dt：                 报告日期
            is_outdate：                是否超期
        """
        self._data = pd.DataFrame()
        
        self._data = self.__parsePage(cf.CLOSED_BOND_FUND_URL, cf.CLOSED_BOND_FUND_COLS)
        self._data[self._data=="-"] = np.NaN
        for col in ['left_year', 'est_val', 'discount_rt', 'annual_discount_rt', 'trade_price',
                    'increase_rt', 'volume', 'fund_nav', 'price_incr_rt', 'stock_ratio', 'bond_ratio']:
            self._data[col] = self._data[col].astype(float)
            
        return self._result()
    
    
    def AHRatio(self):
        """
        A/H比价
        return
        ------
        DataFrame or List: [{'a_code':, 'stock_name':, ...}, ...]
            a_code:                A股代码
            stock_name:            股票名称
            a_price:               A股价格
            a_increase_rt：        A股涨跌幅（％）
            h_code:                H股代码
            h_price:               H股价格（港元）
            h_increase_rt：        H股涨跌幅（％）
            last_time:            最后时间
            rmb_price：            H股价格（人民币）
            hk_currency：          港元汇率
            ha_ratio：             比价（H/A）
            h_free_shares:        H股自由流通市值（亿港元）
            a_free_shares：        A股自由流通市值（亿元）
        """
        self._data = pd.DataFrame()
        
        self._data = self.__parsePage(cf.AH_RATIO_URL, cf.AH_RATIO_COLS)
        self._data[self._data=="-"] = np.NaN
        for col in ['a_price', 'a_increase_rt', 'h_price', 'h_increase_rt', 'rmb_price',
                    'hk_currency', 'ha_ratio', 'h_free_shares', 'a_free_shares']:
            self._data[col] = self._data[col].astype(float)
            
        return self._result()
    
    
    def dividendRate(self):
        """
        A股股息率
        return
        ------
        DataFrame or List: [{'stock_id':, 'stock_nm':, ...}, ...]
            stock_id:                            股票代码
            stock_nm:                            股票名称
            dividend_rate:                       股息率（TTM）
            dividend_rate2：                     静态股息率
            ipo_date：                           ipo日期
            price：                              价格
            volume：                             成交额（万元）
            increase_rt：                        涨幅（％）
            pe:                                  市盈率（TTM）
            pb:                                  市净率
            total_value：                        市值（亿元）
            eps_growth_ttm：                     净利同比增长（％）
            roe:                                 最新年报ROE（％）
            revenue_average：                    5年营收复合增长（％）
            profit_average：                     5年利润复合增长（％）
            roe_average：                        5年平均ROE（％）
            pb_temperature：                     PB温度（℃）
            pe_temperature：                     PE温度（℃）
            int_debt_rate：                      有息负债率（％）
            cashflow_average：                    5年现金流复合增长（％）
            dividend_rate_average：               5年分红率复合增长（％）
            dividend_rate5：                      5年平均股息率（％）
            industry_nm：                         所属行业
            active_flg:                          活跃标志
            last_time:                           最后时间
        """
        self._data = pd.DataFrame()
        
        self._data = self.__parsePage(cf.DIVIDEND_RATE_URL, cf.DIVIDEND_RATE_COLS)
        self._data[self._data=="-"] = np.NaN
        for col in ['dividend_rate', 'dividend_rate2', 'price', 'volume', 'increase_rt', 'pe', 'pb', 'total_value',
                    'eps_growth_ttm', 'roe', 'revenue_average', 'profit_average', 'roe_average', 'pb_temperature',
                    'pe_temperature', 'int_debt_rate', 'cashflow_average', 'dividend_rate_average', 'dividend_rate5']:
            self._data[col] = self._data[col].astype(float)
            
        return self._result()
    
    
    def stockLof(self):
        """
                            股票LOF基金及基相关数据
        return
        ------
        DataFrame or List: [{'fund_id':, 'fund_nm':, ...}, ...]
            fund_id:                基金代码
            fund_nm:                基金名称
            price:                  现价  
            increase_rt:            涨幅（％）
            volume:                 成交（万元）
            amount:                 场内份额（万份）
            fund_nav:               基金净值
            nav_dt:                 净值日期
            estimate_value:         实时估值
            discount_rt:            溢价率（％）
            stock_ratio:            股票占比（％）
            stock_increase_rt:      重仓涨幅（％）
            apply_fee:              申购费（％）
            redeem_fee:             赎回费（％）
            apply_redeem_status:    申赎状态
        """
        self._data = pd.DataFrame()
        
        self._data = self.__parsePage(cf.STOCK_LOF_URL, cf.STOCK_LOF_COLS)
        for col in ['price', 'increase_rt', 'volume', 'amount', 'fund_nav', 
                    'estimate_value', 'discount_rt', 'stock_ratio', 'stock_increase_rt', 'apply_fee', 'redeem_fee']:
            self._data[col] = self._data[col].astype(float)
        
        return self._result()
    
    
    def indexLof(self):
        """
                            指数LOF基金及基相关数据
        return
        ------
        DataFrame or List: [{'fund_id':, 'fund_nm':, ...}, ...]
            fund_id:                基金代码
            fund_nm:                基金名称
            price:                  现价  
            increase_rt:            涨幅（％）
            volume:                 成交（万元）
            amount:                 场内份额（万份）
            fund_nav:               基金净值
            nav_dt:                 净值日期
            estimate_value:         实时估值
            discount_rt:            溢价率（％）
            index_id:               指数代码
            index_nm:               指数名称
            index_increase_rt:      指数涨幅（％）
            apply_fee:              申购费（％）
            redeem_fee:             赎回费（％）
            apply_redeem_status:    申赎状态
        """
        self._data = pd.DataFrame()
        
        self._data = self.__parsePage(cf.INDEX_LOF_URL, cf.INDEX_LOF_COLS)
        for col in ['price', 'increase_rt', 'volume', 'amount', 'fund_nav', 'estimate_value', 
                    'discount_rt', 'index_increase_rt', 'apply_fee', 'redeem_fee']:
            self._data[col] = self._data[col].astype(float)
        
        return self._result()
    
    
    def __parsePage(self, url, column):            
        page = 1
        while(True):
            try:
                request = self._session.get(url % page)
                text = request.text.replace('%', '')
                dataDict = json.loads(text)
                if dataDict['page'] < page:
                    break
                
                dataList = []
                for row in dataDict['rows']:
                    dataList.append(row['cell'])
                  
                self._data = self._data.append(pd.DataFrame(dataList, columns = column), ignore_index=True)
                
                page += 1
            except Exception as e:
                print(str(e))
            
        return self._data
    
    


# -*- coding:utf-8 -*-

__version__ = '0.1.6'
__author__ = 'TabQ'


from gugu.base import Base

from gugu.billboard import BillBoard

from gugu.classify import Classify

from gugu.lowriskintarb import LowRiskIntArb

from gugu.macro import Macro

from gugu.marketdata import MarketData

from gugu.reference import Reference

from gugu.stockdata import StockData

from gugu.stockinfo import StockInfo

from gugu.utility import Utility




# -*- coding:utf-8 -*-
"""
基础类
Created on 2018/12/27
@author: TabQ
@group : gugu
@contact: 16621596@qq.com
"""

import pandas as pd
import sys
import requests
from requests.adapters import HTTPAdapter
from urllib3.util import Retry
import gugu.config as cf


class Base():
    def __init__(self, pandas=True, inter=True):
        self.__pandas = False if not pandas else True
        self._PY3 = (sys.version_info[0] >= 3)
        self._inter = False if not inter else True      # 交互模式
        
        self._session = requests.Session()
        retry = Retry(connect=5, backoff_factor=1)
        adapter = HTTPAdapter(max_retries=retry)
        self._session.mount('http://', adapter)
        self._session.mount('https://', adapter)
        self._session.keep_alive = False
        
        self._data = pd.DataFrame()


    def _result(self):
        """
        返回结果：使用pandas时返回DataFrame否则返回list
        Parameters
        -----
            dataframe:pandas.DataFrame
            index:string
                    返回dict时选用的键 e.g. code
        return
        -----
            pandas.DataFrame or dict
        """
        if self._data.empty:
            return None
        elif self.__pandas:
            return self._data
        else:
            return self._data.to_dict('records')
        
        
    def output(self, full=False):
        print('')
        
        if not full:
            print(self._data)
        else:
            pd.set_option('display.max_columns', None)
            pd.set_option('display.max_rows', None)
            pd.set_option('max_colwidth',100)
            
            print(self._data)
        
        
    def getPandas(self):
        return self.__pandas
    
    
    def setPandas(self, pandas=False):
        self.__pandas = pandas
        
        
    def getInter(self):
        """
        获取交互模式状态
        return
        ------
            True or False
        """
        return self._inter
    
    
    def setInter(self, inter=False):
        """
        设置交互模式
        Parameters
        ------
            inter: bool
        """
        self._inter = inter
        
        
    def _writeHead(self):
        if self._inter:
            sys.stdout.write(cf.GETTING_TIPS)
            sys.stdout.flush()


    def _writeConsole(self):
        if self._inter:
            sys.stdout.write(cf.GETTING_FLAG)
            sys.stdout.flush()  
        
        



#!/usr/bin/env python

from setuptools import setup, find_packages

try:
    import pypandoc
    long_description = pypandoc.convert('README.md', 'rst')
except (IOError, ImportError):
    long_description = ''


setup(
    name='gugu',
    version='0.1.6',
    description='an api for getting financial data',
    long_description=long_description,
    author='TabQ',
    author_email='16621596@qq.com',
    url='http://www.infodata.cc',
    license="Apache License, Version 2.0",
    packages=find_packages(),
    install_requires=[
        'pandas',
        'requests',
        'lxml',
        'simplejson',
        'xlrd'
    ],
    keywords='Financial Data Crawler',
    classifiers=['Development Status :: 4 - Beta',
    'Programming Language :: Python :: 2.6',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3.2',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5'],
)



# -*- coding:utf-8 -*-
"""
龙虎榜类
Created on 2019/01/11
@author: TabQ
@group : gugu
@contact: 16621596@qq.com
"""

import time
import re
import pandas as pd
from pandas.compat import StringIO
import lxml.html
from lxml import etree
from gugu.base import Base, cf
from gugu.utility import Utility

class BillBoard(Base):
    def topList(self, date = None, retry=3, pause=0.001):
        """
        获取每日龙虎榜列表
        Parameters
        --------
        date:string
                    明细数据日期 format：YYYY-MM-DD 如果为空，返回最近一个交易日的数据
        retry : int, 默认 3
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
        
        Return
        ------
        DataFrame or List: [{'code':, 'name':, ...}, ...]
            code：代码
            name ：名称
            pchange：涨跌幅     
            amount：龙虎榜成交额(万)
            buy：买入额(万)
            bratio：占总成交比例
            sell：卖出额(万)
            sratio ：占总成交比例
            reason：上榜原因
            unscramble: 解读
            date  ：日期
        """
        self._data = pd.DataFrame()
        
        if date is None:
            if Utility.getHour() < 18:
                date = Utility.lastTradeDate()
            else:
                date = Utility.getToday()
        else:
            if not Utility.isTradeDay(date):
                return None
            
        for _ in range(retry):
            time.sleep(pause)
            
            try:
                # http://data.eastmoney.com/DataCenter_V3/stock2016/TradeDetail/pagesize=200,page=1,sortRule=-1,sortType=,startDate=2019-01-10,endDate=2019-01-10,gpfw=0,js=vardata_tab_1.html
                request = self._session.get( cf.LHB_URL % (date, date), timeout=10 )
                request.encoding = 'gbk'
                text = request.text.split('_1=')[1]
                dataDict = Utility.str2Dict(text)
                
                self._data = pd.DataFrame(dataDict['data'], columns=cf.LHB_TMP_COLS)
                self._data.columns = cf.LHB_COLS
                self._data['buy'] = self._data['buy'].astype(float)
                self._data['sell'] = self._data['sell'].astype(float)
                self._data['amount'] = self._data['amount'].astype(float)
                self._data['Turnover'] = self._data['Turnover'].astype(float)
                self._data['bratio'] = self._data['buy'] / self._data['Turnover']
                self._data['sratio'] = self._data['sell'] / self._data['Turnover']
                self._data['bratio'] = self._data['bratio'].map(cf.FORMAT)
                self._data['sratio'] = self._data['sratio'].map(cf.FORMAT)
                self._data['date'] = date
                for col in ['amount', 'buy', 'sell']:
                    self._data[col] = self._data[col].astype(float)
                    self._data[col] = self._data[col] / 10000
                    self._data[col] = self._data[col].map(cf.FORMAT)
                self._data = self._data.drop('Turnover', axis=1)
            except:
                pass
            else:
                return self._result()
            
        raise IOError(cf.NETWORK_URL_ERROR_MSG)
    
    
    def countTops(self, days=5, retry=3, pause=0.001):
        """
        获取个股上榜统计数据
        Parameters
        --------
            days:int
                      天数，统计n天以来上榜次数，默认为5天，其余是10、30、60
            retry : int, 默认 3
                         如遇网络等问题重复执行的次数 
            pause : int, 默认 0
                        重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
        Return
        ------
        DataFrame or List: [{'code':, 'name':, ...}, ...]
            code：代码
            name：名称
            count：上榜次数
            bamount：累积购买额(万)     
            samount：累积卖出额(万)
            net：净额(万)
            bcount：买入席位数
            scount：卖出席位数
        """
        self._data = pd.DataFrame()
        
        if Utility.checkLhbInput(days) is True:
            self._writeHead()
            
            # http://vip.stock.finance.sina.com.cn/q/go.php/vLHBData/kind/ggtj/index.phtml?last=5&p=1
            self._data =  self.__parsePage(kind=cf.LHB_KINDS[0], last=days, column=cf.LHB_GGTJ_COLS, dataArr=pd.DataFrame(), pageNo=1, retry=retry, pause=pause)
            self._data['code'] = self._data['code'].map(lambda x: str(x).zfill(6))
            if self._data is not None:
                self._data = self._data.drop_duplicates('code')
                
            return self._result()
        
        
    def brokerTops(self, days=5, retry=3, pause=0.001):
        """
        获取营业部上榜统计数据
        Parameters
        --------
        days:int
                  天数，统计n天以来上榜次数，默认为5天，其余是10、30、60
        retry : int, 默认 3
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
        Return
        ---------
        DataFrame or List: [{'broker':, 'count':, ...}, ...]
            broker：营业部名称
            count：上榜次数
            bamount：累积购买额(万)
            bcount：买入席位数
            samount：累积卖出额(万)
            scount：卖出席位数
            top3：买入前三股票
        """
        self._data = pd.DataFrame()
        
        if Utility.checkLhbInput(days) is True:
            self._writeHead()
            
            # http://vip.stock.finance.sina.com.cn/q/go.php/vLHBData/kind/yytj/index.phtml?last=5&p=1
            self._data = self.__parsePage(kind=cf.LHB_KINDS[1], last=days, column=cf.LHB_YYTJ_COLS, dataArr=pd.DataFrame(), pageNo=1, retry=retry, pause=pause)
            
            return self._result()
        
        
    def instTops(self, days=5, retry=3, pause=0.001):
        """
        获取机构席位追踪统计数据
        Parameters
        --------
        days:int
                  天数，统计n天以来上榜次数，默认为5天，其余是10、30、60
        retry : int, 默认 3
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
                    
        Return
        --------
        DataFrame or List: [{'code':, 'name':, ...}, ...]
            code:代码
            name:名称
            bamount:累积买入额(万)
            bcount:买入次数
            samount:累积卖出额(万)
            scount:卖出次数
            net:净额(万)
        """
        self._data = pd.DataFrame()
        
        if Utility.checkLhbInput(days) is True:
            self._writeHead()
            
            # http://vip.stock.finance.sina.com.cn/q/go.php/vLHBData/kind/jgzz/index.phtml?last=5&p=1
            self._data = self.__parsePage(kind=cf.LHB_KINDS[2], last=days, column=cf.LHB_JGZZ_COLS, dataArr=pd.DataFrame(), pageNo=1, retry=retry, pause=pause, drop_column=[2,3])
            self._data['code'] = self._data['code'].map(lambda x: str(x).zfill(6))
            
            return self._result()
        
        
    def instDetail(self, retry=3, pause=0.001):
        """
        获取最近一个交易日机构席位成交明细统计数据
        Parameters
        --------
        retry : int, 默认 3
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
                    
        Return
        ----------
        DataFrame or List: [{'code':, 'name':, ...}, ...]
            code:股票代码
            name:股票名称     
            date:交易日期     
            bamount:机构席位买入额(万)     
            samount:机构席位卖出额(万)     
            type:类型
        """
        self._data = pd.DataFrame()
        
        self._writeHead()
        
        # http://vip.stock.finance.sina.com.cn/q/go.php/vLHBData/kind/jgmx/index.phtml?last=&p=1
        self._data = self.__parsePage(kind=cf.LHB_KINDS[3], last='', column=cf.LHB_JGMX_COLS, dataArr=pd.DataFrame(), pageNo=1, retry=retry, pause=pause)
        if len(self._data) > 0:
            self._data['code'] = self._data['code'].map(lambda x: str(x).zfill(6))
            
        return self._result()
        
    
    def __parsePage(self, kind, last, column, dataArr, pageNo=1, retry=3, pause=0.001, drop_column=None):
        self._writeConsole()
        
        for _ in range(retry):
            time.sleep(pause)
            
            try:
                request = self._session.get( cf.LHB_SINA_URL % (kind, last, pageNo), timeout=10 )
                request.encoding = 'gbk'
                html = lxml.html.parse(StringIO(request.text))
                res = html.xpath("//table[@id=\"dataTable\"]/tr")
                if self._PY3:
                    sarr = [etree.tostring(node).decode('utf-8') for node in res]
                else:
                    sarr = [etree.tostring(node) for node in res]
                sarr = ''.join(sarr)
                sarr = '<table>%s</table>'%sarr
                df = pd.read_html(sarr)[0]
                if drop_column is not None:
                    df = df.drop(drop_column, axis=1)
                df.columns = column
                dataArr = dataArr.append(df, ignore_index=True)
                nextPage = html.xpath('//div[@class=\"pages\"]/a[last()]/@onclick')
                if len(nextPage) > 0:
                    pageNo = re.findall(r'\d+', nextPage[0])[0]
                    return self.__parsePage(kind, last, column, dataArr, pageNo, retry, pause, drop_column)
                else:
                    return dataArr
            except Exception as e:
                print(e)
                
                



# -*- coding:utf-8 -*-
"""
股票交易类
Created on 2018/12/26
@author: TabQ
@group : gugu
@contact: 16621596@qq.com
"""
from __future__ import division

import time
import json
import re
import lxml.html
from lxml import etree
import pandas as pd
from pandas.compat import StringIO
from gugu.base import Base, cf
from gugu.utility import Utility

class StockData(Base):
    def __init__(self, code=None, pandas=True, inter=True):
        Base.__init__(self, pandas, inter)
        self.__code = code
        
    
    def history(self, start='', end='', ktype='D', autype='qfq', index=False, retry=3, pause=0.001):
        """
        获取股票交易历史数据
        ---------
        Parameters:
          start:string
                      开始日期 format：YYYY-MM-DD 为空时取上市首日
          end:string
                      结束日期 format：YYYY-MM-DD 为空时取最近一个交易日
          autype:string
                      复权类型，qfq-前复权 hfq-后复权 None-不复权，默认为qfq
          ktype：string
                      数据类型，D=日k线 W=周 M=月 5=5分钟 15=15分钟 30=30分钟 60=60分钟，默认为D
          retry: int, 默认 3
                     如遇网络等问题重复执行的次数 
          pause : int, 默认 0
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
        return
        -------
          DataFrame or list: [{'date':, 'open':, ...}, ...]
              date 交易日期 (index)
              open 开盘价
              high  最高价
              close 收盘价
              low 最低价
              volume 成交量
              code 股票代码
        """
        self._data = pd.DataFrame()
        
        url = ''
        dataflag = ''
        symbol = cf.INDEX_SYMBOL[self.__code] if index else Utility.symbol(self.__code)
        autype = '' if autype is None else autype
        
        if (start is not None) & (start != ''):
            end = Utility.getToday() if end is None or end == '' else end
            
        if ktype.upper() in cf.K_LABELS:
            fq = autype if autype is not None else ''
            if self.__code[:1] in ('1', '5') or index:
                fq = ''
                
            kline = '' if autype is None else 'fq'
            if (start is None or start == '') or (end is None or end == ''):
                # http://web.ifzq.gtimg.cn/appstock/app/fqkline/get?_var=kline_dayqfq&param=sh600000,day,,,640,qfq&r=0.73327461855388564
                urls = [cf.HISTORY_URL % (kline, fq, symbol, cf.TT_K_TYPE[ktype.upper()], start, end, fq, Utility.random(17))]
            else:
                years = Utility.ttDates(start, end)
                urls = []
                for year in years:
                    startdate = str(year) + '-01-01'
                    enddate = str(year+1) + '-12-31'
                    # http://web.ifzq.gtimg.cn/appstock/app/fqkline/get?_var=kline_dayqfq2008&param=sh600000,day,2008-01-01,2009-12-31,640,qfq&r=0.73327461855388564
                    url = cf.HISTORY_URL % (kline, fq+str(year), symbol, cf.TT_K_TYPE[ktype.upper()], startdate, enddate, fq, Utility.random(17))
                    urls.append(url)
            dataflag = '%s%s'%(fq, cf.TT_K_TYPE[ktype.upper()])     # qfqday
        elif ktype in cf.K_MIN_LABELS:
            # http://ifzq.gtimg.cn/appstock/app/kline/mkline?param=sh600000,m30,,640&_var=m30_today&r=0.5537154641907898
            urls = [cf.HISTORY_MIN_URL % (symbol, ktype, ktype, Utility.random(16))]
            dataflag = 'm%s'%ktype      # m30
        else:
            raise TypeError('ktype input error.')
        
        for url in urls:
            self._data = self._data.append(self.__handleHistory(url, dataflag, symbol, index, ktype, retry, pause), ignore_index=True)
        if ktype not in cf.K_MIN_LABELS:
            if ((start is not None) & (start != '')) & ((end is not None) & (end != '')):
                self._data = self._data[(self._data.date >= start) & (self._data.date <= end)]
        
        return self._result()
    
        raise IOError(cf.NETWORK_URL_ERROR_MSG)
    
    
    def __handleHistory(self, url, dataflag='', symbol='', index = False, ktype = '', retry=3, pause=0.001):
        for _ in range(retry):
            time.sleep(pause)
            
            try:
                request = self._session.get(url, timeout=10)
                if self._PY3:
                    request.encoding = 'gbk'
                lines = request.text
                if len(lines) < 100: #no data
                    return None
            except Exception as e:
                print(e)
            else:
                lines = lines.split('=')[1]
                reg = re.compile(r',{"nd.*?}') 
                lines = re.subn(reg, '', lines) 
                js = json.loads(lines[0])
                dataflag = dataflag if dataflag in list(js['data'][symbol].keys()) else cf.TT_K_TYPE[ktype.upper()]
                
                if ktype in cf.K_MIN_LABELS:
                    for value in js['data'][symbol][dataflag]:
                        value.pop()
                        value.pop()
                        
                df = pd.DataFrame(js['data'][symbol][dataflag], columns=cf.KLINE_TT_COLS)
                df['code'] = symbol if index else self.__code
                if ktype in cf.K_MIN_LABELS:
                    df['date'] = df['date'].map(lambda x: '%s-%s-%s %s:%s'%(x[0:4], x[4:6], x[6:8], x[8:10], x[10:12]))
                for col in df.columns[1:6]:
                    df[col] = df[col].astype(float)
                    
                return df
            
            
    def xrxd(self, date='', retry=3, pause=0.001):
        """
        获取股票除权除息信息
        Parameters
        ------
            date: string
                format：YYYY-MM-DD
            retry: int, 默认 3
                如遇网络等问题重复执行的次数 
            pause : int, 默认 0
                重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
                
        return
        ------
            Dict or None
                nd, 对应年份
                fh_sh, 分红数额
                djr, 股权登记日
                cqr, 除权日
                FHcontent, 除权除息信息
        """
        data = None
        
        if not date:
            date = Utility.getToday()
            
        symbol = Utility.symbol(self.__code)
        
        for _ in range(retry):
            time.sleep(pause)
            
            url = cf.HISTORY_URL % ('fq', 'qfq', symbol, 'day', date, date, 'qfq', Utility.random(17))
            try:
                request = self._session.get(url, timeout=10)
                pattern = re.compile(r'({"nd".+?})')
                result = re.search(pattern, request.text)
                if result:
                    data = eval(result.group(1))
            except Exception as e:
                print(str(e))
                
            return data
                
        raise IOError(cf.NETWORK_URL_ERROR_MSG)
    
    
    def realtime(self):
        """
        获取实时交易数据 getting real time quotes data
        用于跟踪交易情况（本次执行的结果-上一次执行的数据）
        Parameters
        ------
            stockdata(code) code : string, array-like object (list, tuple, Series).
        return
        -------
            DataFrame or list: [{'name':, 'open':, ...}, ...]
            	0：name，股票名字
                1：open，今日开盘价
                2：pre_close，昨日收盘价
                3：price，当前价格
                4：high，今日最高价
                5：low，今日最低价
                6：bid，竞买价，即“买一”报价
                7：ask，竞卖价，即“卖一”报价
                8：volumn，成交量 maybe you need do volumn/100
                9：amount，成交金额（元 CNY）
                10：b1_v，委买一（笔数 bid volume）
                11：b1_p，委买一（价格 bid price）
                12：b2_v，“买二”
                13：b2_p，“买二”
                14：b3_v，“买三”
                15：b3_p，“买三”
                16：b4_v，“买四”
                17：b4_p，“买四”
                18：b5_v，“买五”
                19：b5_p，“买五”
                20：a1_v，委卖一（笔数 ask volume）
                21：a1_p，委卖一（价格 ask price）
                ...
                30：date，日期；
                31：time，时间；
        """
        self._data = pd.DataFrame()
        
        symbols_list = ''
        if isinstance(self.__code, list) or isinstance(self.__code, set) or isinstance(self.__code, tuple) or isinstance(self.__code, pd.Series):
            for code in self.__code:
                symbols_list += Utility.symbol(code) + ','
        else:
            symbols_list = Utility.symbol(self.__code)
        symbols_list = symbols_list[:-1] if len(symbols_list) > 8 else symbols_list
        
        # http://hq.sinajs.cn/rn=4879967949085&list=sh600000,sh600004
        request = self._session.get( cf.LIVE_DATA_URL % (Utility.random(), symbols_list), timeout=10 )
        request.encoding = 'gbk'
        reg = re.compile(r'\="(.*?)\";')
        data = reg.findall(request.text)
        regSym = re.compile(r'(?:sh|sz)(.*?)\=')
        syms = regSym.findall(request.text)
        data_list = []
        syms_list = []
        for index, row in enumerate(data):
            if len(row)>1:
                data_list.append([astr for astr in row.split(',')])
                syms_list.append(syms[index])
        if len(syms_list) == 0:
            return None
        
        self._data = pd.DataFrame(data_list, columns=cf.LIVE_DATA_COLS)
        self._data = self._data.drop('s', axis=1)
        self._data['code'] = syms_list
        ls = [cls for cls in self._data.columns if '_v' in cls]
        for txt in ls:
            self._data[txt] = self._data[txt].map(lambda x : x[:-2])
            
        return self._result()
    
    
    def historyTicks(self, date=None, retry=3, pause=0.001):
        """
        获取历史分笔明细数据
        Parameters
        ------
            date : string
                    日期 format：YYYY-MM-DD, 默认为前一个交易日
            retry : int, 默认 3
                    如遇网络等问题重复执行的次数
            pause : int, 默认 0
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
         return
         -------
            DataFrame or list: [{'time':, 'price':, ...}, ...]
		time：时间
		price：成交价格
		change：价格变动
		volume：成交手
		amount：成交金额(元)
		type：买卖类型【买盘、卖盘、中性盘】
        """
        self._data = pd.DataFrame()
        
        symbol = Utility.symbol(self.__code)
        date = Utility.lastTradeDate() if date is None else date
            
        try:
            self._writeHead()
            
            page = 1
            while(True):
                # http://vip.stock.finance.sina.com.cn/quotes_service/view/vMS_tradehistory.php?symbol=sh600000&date=2018-12-26&page=1
                # http://market.finance.sina.com.cn/transHis.php?date=2019-01-25&symbol=sh600000&page=1
                url = cf.HISTORY_TICKS_URL % (date, symbol, page)
                tick_data = self.__handleTicks(url, cf.HISTORY_TICK_COLUMNS, retry, pause)
                if tick_data is not None:
                    self._data = self._data.append(tick_data, ignore_index=True)
                    page = page + 1
                else:
                    break
        except Exception as er:
            print(str(er))
        else:
            return self._result()
            
        raise IOError(cf.NETWORK_URL_ERROR_MSG)
        
        
    def __handleTicks(self, url, column, retry, pause):
        self._writeConsole()
        
        for _ in range(retry):
            time.sleep(pause)
            
            try:
                html = lxml.html.parse(url)
                res = html.xpath('//table[@id=\"datatbl\"]/tbody/tr')
                if not res:
                    return None
                
                if self._PY3:
                    sarr = [etree.tostring(node).decode('utf-8') for node in res]
                else:
                    sarr = [etree.tostring(node) for node in res]
                sarr = ''.join(sarr)
                sarr = '<table>%s</table>'%sarr
                sarr = sarr.replace('--', '0')
                df = pd.read_html(StringIO(sarr), parse_dates=False)
                df = pd.read_html(StringIO(sarr), parse_dates=False)[0]
                df.columns = column
                if 'pchange' in column:
                    df['pchange'] = df['pchange'].map(lambda x : x.replace('%', ''))
            except Exception as e:
                print(e)
            else:
                return df
            
        raise IOError(cf.NETWORK_URL_ERROR_MSG)
    
    def todayTicks(self, retry=3, pause=0.001):
        """
        获取当日分笔明细数据
        Parameters
        ------
            retry : int, 默认 3
                      如遇网络等问题重复执行的次数
            pause : int, 默认 0
                     重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
         return
         -------
            DataFrame or list: [{'time':, 'price':, ...}, ...]      
		    time：时间
		    price：当前价格
		    pchange:涨跌幅
		    change：价格变动
		    volume：成交手
		    amount：成交金额(元)
		    type：买卖类型【买盘、卖盘、中性盘】

        """
        self._data = pd.DataFrame()
        
        if self.__code is None or len(self.__code)!=6 :
            return None
        
        if not Utility.isTradeDay():
            return None
        
        # 不到交易时间
        openTime = time.mktime(time.strptime(Utility.getToday() + ' 09:25:00', '%Y-%m-%d %H:%M:%S'))
        now = time.time()
        if now < openTime:
            return None
        
        symbol = Utility.symbol(self.__code)
        date = Utility.getToday()
        
        for _ in range(retry):
            time.sleep(pause)
            
            try:
                # http://vip.stock.finance.sina.com.cn/quotes_service/api/json_v2.php/CN_Transactions.getAllPageTime?date=2018-12-26&symbol=sh600000
                request = self._session.get( cf.TODAY_TICKS_PAGE_URL % (date, symbol), timeout=10 )
                request.encoding = 'gbk'
                text = request.text[1:-1]
                data_dict = Utility.str2Dict(text)
                pages = len(data_dict['detailPages'])
                
                self._writeHead()
                for pNo in range(1, pages+1):
                    # http://vip.stock.finance.sina.com.cn/quotes_service/view/vMS_tradedetail.php?symbol=sh600000&date=2018-12-26&page=1
                    url = cf.TODAY_TICKS_URL % (symbol, date, pNo)
                    self._data = self._data.append(self.__handleTicks(url, cf.TODAY_TICK_COLUMNS, retry, pause), ignore_index=True)
            except Exception as e:
                print(str(e))
            else:
                return self._result()
            
        raise IOError(cf.NETWORK_URL_ERROR_MSG)
    
    
    def bigDeal(self, date=None, vol=400, retry=3, pause=0.001):
        """
        获取大单数据
        Parameters
        ------
            date:string
                      日期 format：YYYY-MM-DD
            retry : int, 默认 3
                      如遇网络等问题重复执行的次数
            pause : int, 默认 0
                     重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
         return
         -------
            DataFrame or list: [{'code':, 'name', ...}, ...]
		    code：代码
		    name：名称
		    time：时间
		    price：当前价格
		    volume：成交手
		    preprice ：上一笔价格
		    type：买卖类型【买盘、卖盘、中性盘】
        """
        self._data = pd.DataFrame()
        
        if self.__code is None or len(self.__code) != 6 or date is None:
            return None
        
        symbol = Utility.symbol(self.__code)
        vol = vol*100
        
        for _ in range(retry):
            time.sleep(pause)
            
            try:
                # http://vip.stock.finance.sina.com.cn/quotes_service/view/cn_bill_download.php?symbol=sh600000&num=60&page=1&sort=ticktime&asc=0&volume=40000&amount=0&type=0&day=2018-12-26
                request = self._session.get( cf.SINA_DD % (symbol, vol, date), timeout=10 )
                request.encoding = 'gbk'
                lines = request.text
                if len(lines) < 100:
                    return None
                self._data = pd.read_csv(StringIO(lines), names=cf.SINA_DD_COLS, skiprows=[0])    
                if self._data is not None:
                    self._data['code'] = self._data['code'].map(lambda x: x[2:])
            except Exception as e:
                print(e)
            else:
                return self._result()
            
        raise IOError(cf.NETWORK_URL_ERROR_MSG)
    
    



# -*- coding:utf-8 -*-
"""
宏观经济数据类
Created on 2019/01/09
@author: TabQ
@group : gugu
@contact: 16621596@qq.com
"""
import pandas as pd
import numpy as np
import re
import json
import time
from gugu.utility import Utility
from gugu.base import Base, cf


class Macro(Base):
    def gdpYear(self, retry=3, pause=0.001):
        """
        获取年度国内生产总值数据
        Parameters
        --------
        retry : int, 默认 3
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0.001
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
                    
        Return
        --------
        DataFrame or List: [{'year':, 'gdp':, ...}, ...]
            year :统计年度
            gdp :国内生产总值(亿元)
            pc_gdp :人均国内生产总值(元)
            gnp :国民生产总值(亿元)
            pi :第一产业(亿元)
            si :第二产业(亿元)
            industry :工业(亿元)
            cons_industry :建筑业(亿元)
            ti :第三产业(亿元)
            trans_industry :交通运输仓储邮电通信业(亿元)
            lbdy :批发零售贸易及餐饮业(亿元)
        """
        self._data = pd.DataFrame()
        
        # http://money.finance.sina.com.cn/mac/api/jsonp.php/SINAREMOTECALLCALLBACK4224641560861/MacPage_Service.get_pagedata?cate=nation&event=0&from=0&num=70&condition=&_=4224641560861
        datastr = self.__parsePage('nation', 0, 70, retry, pause)
        datastr = datastr.replace('"', '').replace('null', '0')
        js = json.loads(datastr)
        self._data = pd.DataFrame(js, columns=cf.GDP_YEAR_COLS)
        self._data[self._data==0] = np.NaN
        
        return self._result()
    
    def gdpQuarter(self, retry=3, pause=0.001):
        """
        获取季度国内生产总值数据
        Parameters
        --------
        retry : int, 默认 3
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0.001
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
                    
        Return
        --------
        DataFrame or List: [{'quarter':, 'gdp':, ...}, ...]
            quarter :季度
            gdp :国内生产总值(亿元)
            gdp_yoy :国内生产总值同比增长(%)
            pi :第一产业增加值(亿元)
            pi_yoy:第一产业增加值同比增长(%)
            si :第二产业增加值(亿元)
            si_yoy :第二产业增加值同比增长(%)
            ti :第三产业增加值(亿元)
            ti_yoy :第三产业增加值同比增长(%)
        """
        self._data = pd.DataFrame()
        
        # http://money.finance.sina.com.cn/mac/api/jsonp.php/SINAREMOTECALLCALLBACK3935140379887/MacPage_Service.get_pagedata?cate=nation&event=1&from=0&num=250&condition=&_=3935140379887
        datastr = self.__parsePage('nation', 1, 250, retry, pause)
        datastr = datastr.replace('"', '').replace('null', '0')
        js = json.loads(datastr)
        self._data = pd.DataFrame(js, columns=cf.GDP_QUARTER_COLS)
        self._data['quarter'] = self._data['quarter'].astype(object)
        self._data[self._data==0] = np.NaN
        
        return self._result()
    
    def demandsToGdp(self, retry=3, pause=0.001):
        """
        获取三大需求对GDP贡献数据
        Parameters
        --------
        retry : int, 默认 3
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0.001
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
                    
        Return
        --------
        DataFrame or List: [{'year':, 'cons_to':, ...}, ...]
            year :统计年度
            cons_to :最终消费支出贡献率(%)
            cons_rate :最终消费支出拉动(百分点)
            asset_to :资本形成总额贡献率(%)
            asset_rate:资本形成总额拉动(百分点)
            goods_to :货物和服务净出口贡献率(%)
            goods_rate :货物和服务净出口拉动(百分点)
        """
        self._data = pd.DataFrame()
        
        # http://money.finance.sina.com.cn/mac/api/jsonp.php/SINAREMOTECALLCALLBACK3153587567694/MacPage_Service.get_pagedata?cate=nation&event=4&from=0&num=80&condition=&_=3153587567694
        datastr = self.__parsePage('nation', 4, 80, retry, pause)
        datastr = datastr.replace('"','').replace('null','0')
        js = json.loads(datastr)
        self._data = pd.DataFrame(js,columns=cf.GDP_FOR_COLS)
        self._data[self._data==0] = np.NaN
        
        return self._result()
        
    
    def idsPullToGdp(self, retry=3, pause=0.001):
        """
        获取三大产业对GDP拉动数据
        Parameters
        --------
        retry : int, 默认 3
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0.001
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
                    
        Return
        --------
        DataFrame or List: [{'year':, 'gdp_yoy':, ...}, ...]
            year :统计年度
            gdp_yoy :国内生产总值同比增长(%)
            pi :第一产业拉动率(%)
            si :第二产业拉动率(%)
            industry:其中工业拉动(%)
            ti :第三产业拉动率(%)
        """
        self._data = pd.DataFrame()
        
        # http://money.finance.sina.com.cn/mac/api/jsonp.php/SINAREMOTECALLCALLBACK1083239038283/MacPage_Service.get_pagedata?cate=nation&event=5&from=0&num=60&condition=&_=1083239038283
        datastr = self.__parsePage('nation', 5, 60, retry, pause)
        datastr = datastr.replace('"', '').replace('null', '0')
        js = json.loads(datastr)
        self._data = pd.DataFrame(js, columns=cf.GDP_PULL_COLS)
        self._data[self._data==0] = np.NaN
        
        return self._result()
    
    
    def idsCtbToGdp(self, retry=3, pause=0.001):
        """
        获取三大产业贡献率数据
        Parameters
        --------
        retry : int, 默认 3
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0.001
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
                    
        Return
        --------
        DataFrame or List: [{'year':, 'gdp_yoy':, ...}, ...]
            year :统计年度
            gdp_yoy :国内生产总值
            pi :第一产业献率(%)
            si :第二产业献率(%)
            industry:其中工业献率(%)
            ti :第三产业献率(%)
        """
        self._data = pd.DataFrame()
        
        # http://money.finance.sina.com.cn/mac/api/jsonp.php/SINAREMOTECALLCALLBACK4658347026358/MacPage_Service.get_pagedata?cate=nation&event=6&from=0&num=60&condition=&_=4658347026358
        datastr = self.__parsePage('nation', 6, 60, retry, pause)
        datastr = datastr.replace('"', '').replace('null', '0')
        js = json.loads(datastr)
        self._data = pd.DataFrame(js, columns=cf.GDP_CONTRIB_COLS)
        self._data[self._data==0] = np.NaN
        
        return self._result()
    
    
    def cpi(self, retry=3, pause=0.001):
        """
        获取居民消费价格指数数据
        Parameters
        --------
        retry : int, 默认 3
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0.001
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
                    
        Return
        --------
        DataFrame or List: [{'month':, 'cpi':,}, ...]
            month :统计月份
            cpi :价格指数
        """
        self._data = pd.DataFrame()
        
        datastr = self.__parsePage('price', 0, 600, retry, pause)
        js = json.loads(datastr)
        self._data = pd.DataFrame(js, columns=cf.CPI_COLS)
        self._data['cpi'] = self._data['cpi'].astype(float)
        
        return self._result()
    
    
    def ppi(self, retry=3, pause=0.001):
        """
        获取工业品出厂价格指数数据
        Parameters
        --------
        retry : int, 默认 3
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0.001
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
                    
        Return
        --------
        DataFrame or List: [{'month':, 'ppiip':, ...}, ...]
            month :统计月份
            ppiip :工业品出厂价格指数
            ppi :生产资料价格指数
            qm:采掘工业价格指数
            rmi:原材料工业价格指数
            pi:加工工业价格指数    
            cg:生活资料价格指数
            food:食品类价格指数
            clothing:衣着类价格指数
            roeu:一般日用品价格指数
            dcg:耐用消费品价格指数
        """
        self._data = pd.DataFrame()
        
        # http://money.finance.sina.com.cn/mac/api/jsonp.php/SINAREMOTECALLCALLBACK6734345383111/MacPage_Service.get_pagedata?cate=price&event=3&from=0&num=600&condition=&_=6734345383111
        datastr = self.__parsePage('price', 3, 600, retry, pause)
        js = json.loads(datastr)
        self._data = pd.DataFrame(js, columns=cf.PPI_COLS)
        for i in self._data.columns:
            self._data[i] = self._data[i].apply(lambda x:np.where(x is None, np.NaN, x))
            if i != 'month':
                self._data[i] = self._data[i].astype(float)
                
        return self._result()
    
    
    def depositRate(self, retry=3, pause=0.001):
        """
        获取存款利率数据
        Parameters
        --------
        retry : int, 默认 3
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0.001
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
                    
        Return
        --------
        DataFrame or List: [{'date':, 'deposit_type':, ...}, ...]
            date :变动日期
            deposit_type :存款种类
            rate:利率（%）
        """
        self._data = pd.DataFrame()
        
        # http://money.finance.sina.com.cn/mac/api/jsonp.php/SINAREMOTECALLCALLBACK1250640915421/MacPage_Service.get_pagedata?cate=fininfo&event=2&from=0&num=600&condition=&_=1250640915421
        datastr = self.__parsePage('fininfo', 2, 600, retry, pause)
        js = json.loads(datastr)
        self._data = pd.DataFrame(js, columns=cf.DEPOSIT_COLS)
        for i in self._data.columns:
            self._data[i] = self._data[i].apply(lambda x:np.where(x is None, '--', x))
            
        return self._result()
    
    
    def loanRate(self, retry=3, pause=0.001):
        """
        获取贷款利率数据
        Parameters
        --------
        retry : int, 默认 3
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0.001
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
                    
        Return
        --------
        DataFrame or List: [{'date':, 'loan_type':, ...}, ...]
            date :执行日期
            loan_type :存款种类
            rate:利率（%）
        """
        self._data = pd.DataFrame()
        
        # http://money.finance.sina.com.cn/mac/api/jsonp.php/SINAREMOTECALLCALLBACK7542659823280/MacPage_Service.get_pagedata?cate=fininfo&event=3&from=0&num=800&condition=&_=7542659823280
        datastr = self.__parsePage('fininfo', 3, 800, retry, pause)
        js = json.loads(datastr)
        self._data = pd.DataFrame(js, columns=cf.LOAN_COLS)
        for i in self._data.columns:
            self._data[i] = self._data[i].apply(lambda x:np.where(x is None, '--', x))
            
        return self._result()
    
    
    def rrr(self, retry=3, pause=0.001):
        """
        获取存款准备金率数据
        Parameters
        --------
        retry : int, 默认 3
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0.001
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
                    
        Return
        --------
        DataFrame or List: [{'date':, 'before':, ...}, ...]
            date :变动日期
            before :调整前存款准备金率(%)
            now:调整后存款准备金率(%)
            changed:调整幅度(%)
        """
        self._data = pd.DataFrame()
        
        # http://money.finance.sina.com.cn/mac/api/jsonp.php/SINAREMOTECALLCALLBACK8028217046046/MacPage_Service.get_pagedata?cate=fininfo&event=4&from=0&num=100&condition=&_=8028217046046
        datastr = self.__parsePage('fininfo', 4, 100, retry, pause)
        datastr = datastr if self._PY3 else datastr.decode('gbk')
        js = json.loads(datastr)
        self._data = pd.DataFrame(js, columns=cf.RRR_COLS)
        for i in self._data.columns:
            self._data[i] = self._data[i].apply(lambda x:np.where(x is None, '--', x))
            
        return self._result()
    
    
    def montySupply(self, retry=3, pause=0.001):
        """
        获取货币供应量数据
        Parameters
        --------
        retry : int, 默认 3
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0.001
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
                    
        Return
        --------
        DataFrame or List: [{'month':, 'm2':, ...}, ...]
            month :统计时间
            m2 :货币和准货币（广义货币M2）(亿元)
            m2_yoy:货币和准货币（广义货币M2）同比增长(%)
            m1:货币(狭义货币M1)(亿元)
            m1_yoy:货币(狭义货币M1)同比增长(%)
            m0:流通中现金(M0)(亿元)
            m0_yoy:流通中现金(M0)同比增长(%)
            cd:活期存款(亿元)
            cd_yoy:活期存款同比增长(%)
            qm:准货币(亿元)
            qm_yoy:准货币同比增长(%)
            ftd:定期存款(亿元)
            ftd_yoy:定期存款同比增长(%)
            sd:储蓄存款(亿元)
            sd_yoy:储蓄存款同比增长(%)
            rests:其他存款(亿元)
            rests_yoy:其他存款同比增长(%)
        """
        self._data = pd.DataFrame()
        
        # http://money.finance.sina.com.cn/mac/api/jsonp.php/SINAREMOTECALLCALLBACK9019314616219/MacPage_Service.get_pagedata?cate=fininfo&event=1&from=0&num=600&condition=&_=9019314616219
        datastr = self.__parsePage('fininfo', 1, 600, retry, pause)
        datastr = datastr if self._PY3 else datastr.decode('gbk')
        js = json.loads(datastr)
        self._data = pd.DataFrame(js, columns=cf.MONEY_SUPPLY_COLS)
        for i in self._data.columns:
            self._data[i] = self._data[i].apply(lambda x:np.where(x is None, '--', x))
            
        return self._result()
    
    
    def moneySupplyBal(self, retry=3, pause=0.001):
        """
        获取货币供应量(年底余额)数据
        Parameters
        --------
        retry : int, 默认 3
                     如遇网络等问题重复执行的次数 
        pause : int, 默认 0.001
                    重复请求数据过程中暂停的秒数，防止请求间隔时间太短出现的问题
                    
        Return
        --------
        DataFrame or List: [{'year':, 'm2':, ...}, ...]
            year :统计年度
            m2 :货币和准货币(亿元)
            m1:货币(亿元)
            m0:流通中现金(亿元)
            cd:活期存款(亿元)
            qm:准货币(亿元)
            ftd:定期存款(亿元)
            sd:储蓄存款(亿元)
            rests:其他存款(亿元)
        """
        self._data = pd.DataFrame()
        
        # http://money.finance.sina.com.cn/mac/api/jsonp.php/SINAREMOTECALLCALLBACK3430820865181/MacPage_Service.get_pagedata?cate=fininfo&event=0&from=0&num=200&condition=&_=3430820865181
        datastr = self.__parsePage('fininfo', 0, 200, retry, pause)
        datastr = datastr if self._PY3 else datastr.decode('gbk')
        js = json.loads(datastr)
        self._data = pd.DataFrame(js, columns=cf.MONEY_SUPPLY_BLA_COLS)
        for i in self._data.columns:
            self._data[i] = self._data[i].apply(lambda x:np.where(x is None, '--', x))
            
        return self._result()
    
    
    def __parsePage(self, cate='', event=0, num=0, retry=3, pause=0.001):
        for _ in range(retry):
            time.sleep(pause)
            
            try:
                rdInt = Utility.random()
                request = self._session.get( cf.MACRO_URL % (rdInt, cate, event, num, rdInt), timeout=10 )
                if self._PY3:
                    request.encoding = 'gbk'
                    
                regSym = re.compile(r'\,count:(.*?)\}')
                datastr = regSym.findall(request.text)
                datastr = datastr[0]
                datastr = datastr.split('data:')[1]
            except Exception as e:
                print(e)
            else:
                return datastr
            
        raise IOError(cf.NETWORK_URL_ERROR_MSG)
    
    
    def shibor(self, year=None):
        """
        获取上海银行间同业拆放利率
        Parameters
        ------
          year:年份(int)
          
        Return
        ------
        DataFrame or List: [{'date':, 'ON':, ...}, ...]
            date:日期
            ON:隔夜拆放利率
            1W:1周拆放利率
            2W:2周拆放利率
            1M:1个月拆放利率
            3M:3个月拆放利率
            6M:6个月拆放利率
            9M:9个月拆放利率
            1Y:1年拆放利率
        """
        self._data = pd.DataFrame()
        
        lab = cf.SHIBOR_TYPE['Shibor']
        # http://www.shibor.org/shibor/web/html/downLoad.html?nameNew=Historical_Shibor_Data_2018.xls&downLoadPath=data&nameOld=Shibor数据2018.xls&shiborSrc=http://www.shibor.org/shibor/
        self._data = self.__parseExcel(year, 'Shibor', lab, cf.SHIBOR_COLS)
        
        return self._result()
        
    
    def shiborQuote(self, year=None):
        """
        获取Shibor银行报价数据
        Parameters
        ------
          year:年份(int)
          
        Return
        ------
        DataFrame or List: [{'date':, 'bank':, ...}, ...]
            date:日期
            bank:报价银行名称
            ON:隔夜拆放利率
            1W:1周拆放利率
            2W:2周拆放利率
            1M:1个月拆放利率
            3M:3个月拆放利率
            6M:6个月拆放利率
            9M:9个月拆放利率
            1Y:1年拆放利率
        """
        self._data = pd.DataFrame()
        
        lab = cf.SHIBOR_TYPE['Quote']
        # http://www.shibor.org/shibor/web/html/downLoad.html?nameNew=Historical_Quote_Data_2018.xls&downLoadPath=data&nameOld=报价数据2018.xls&shiborSrc=http://www.shibor.org/shibor/
        self._data = self.__parseExcel(year, 'Quote', lab, cf.QUOTE_COLS)
        
        return self._result()
    
    
    def shiborMa(self, year=None):
        """
        获取Shibor均值数据
        Parameters
        ------
          year:年份(int)
          
        Return
        ------
        DataFrame or List: [{'date':, 'ON_5':, ...}, ...]
            date:日期
               其它分别为各周期5、10、20均价
        """
        self._data = pd.DataFrame()
        
        lab = cf.SHIBOR_TYPE['Tendency']
        self._data = self.__parseExcel(year, 'Shibor_Tendency', lab, cf.SHIBOR_MA_COLS)
        
        return self._result()
    
    
    def lpr(self, year=None):
        """
        获取贷款基础利率
        Parameters
        ------
          year:年份(int)
          
        Return
        ------
        DataFrame or List: [{'date':, '1Y':, ...}, ...]
            date:日期
            1Y:1年贷款基础利率
        """
        self._data = pd.DataFrame()
        
        lab = cf.SHIBOR_TYPE['LPR']
        self._data = self.__parseExcel(year, 'LPR', lab, cf.LPR_COLS)
        
        return self._result()
    
    
    def lprMa(self, year=None):
        """
        获取贷款基础利率均值数据
        Parameters
        ------
          year:年份(int)
          
        Return
        ------
        DataFrame or List: [{'date':, '1Y_5':, ...}, ...]
            date:日期
            1Y_5:5日均值
            1Y_10:10日均值
            1Y_20:20日均值
        """
        self._data = pd.DataFrame()
        
        lab = cf.SHIBOR_TYPE['LPR_Tendency']
        self._data = self.__parseExcel(year, 'LPR_Tendency', lab, cf.LPR_MA_COLS)
        
        return self._result()
        
        
    def __parseExcel(self, year, datatype, lab, column):
        year = Utility.getYear() if year is None else year
        lab = lab.encode('utf-8') if self._PY3 else lab
        
        try:
            df = pd.read_excel( cf.SHIBOR_DATA_URL % (datatype, year, lab, year), skiprows=[0] )
            df.columns = column
            df['date'] = df['date'].map(lambda x: x.date())
            df['date'] = df['date'].astype('datetime64[ns]')
        except Exception as e:
            print(e)
        else:
            return df
        
        
